from typing import TypedDict, Literal, NotRequired, Any, Coroutine, Union

# Payload TypedDict for IDE autocomplete
class HsmLocalPayload(TypedDict, total=False):
    """
    Type hints for vpn/certificate/hsm_local payload fields.
    
    Use this for IDE autocomplete when building payload dicts:
        payload: HsmLocalPayload = {
            "field": "value",  # <- autocomplete shows all fields
        }
    """
    name: str  # Name.
    comments: NotRequired[str]  # Comment.
    vendor: Literal["unknown", "gch"]  # HSM vendor.
    api_version: NotRequired[Literal["unknown", "gch-default"]]  # API version for communicating with HSM.
    certificate: NotRequired[str]  # PEM format certificate.
    range: NotRequired[Literal["global", "vdom"]]  # Either a global or VDOM IP address range for the certificate
    source: NotRequired[Literal["factory", "user", "bundle"]]  # Certificate source type.
    gch_url: NotRequired[str]  # Google Cloud HSM key URL (e.g. "https://cloudkms.googleapis.
    gch_project: NotRequired[str]  # Google Cloud HSM project ID.
    gch_location: NotRequired[str]  # Google Cloud HSM location.
    gch_keyring: NotRequired[str]  # Google Cloud HSM keyring.
    gch_cryptokey: NotRequired[str]  # Google Cloud HSM cryptokey.
    gch_cryptokey_version: NotRequired[str]  # Google Cloud HSM cryptokey version.
    gch_cloud_service_name: NotRequired[str]  # Cloud service config name to generate access token.
    gch_cryptokey_algorithm: NotRequired[Literal["rsa-sign-pkcs1-2048-sha256", "rsa-sign-pkcs1-3072-sha256", "rsa-sign-pkcs1-4096-sha256", "rsa-sign-pkcs1-4096-sha512", "rsa-sign-pss-2048-sha256", "rsa-sign-pss-3072-sha256", "rsa-sign-pss-4096-sha256", "rsa-sign-pss-4096-sha512", "ec-sign-p256-sha256", "ec-sign-p384-sha384", "ec-sign-secp256k1-sha256"]]  # Google Cloud HSM cryptokey algorithm.
    details: NotRequired[str]  # Print hsm-local certificate detailed information.


class HsmLocal:
    """
    Local certificates whose keys are stored on HSM.
    
    Path: vpn/certificate/hsm_local
    Category: cmdb
    Primary Key: name
    """
    
    def get(
        self,
        name: str | None = ...,
        filter: str | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def post(
        self,
        payload_dict: HsmLocalPayload | None = ...,
        name: str | None = ...,
        comments: str | None = ...,
        vendor: Literal["unknown", "gch"] | None = ...,
        api_version: Literal["unknown", "gch-default"] | None = ...,
        certificate: str | None = ...,
        range: Literal["global", "vdom"] | None = ...,
        source: Literal["factory", "user", "bundle"] | None = ...,
        gch_url: str | None = ...,
        gch_project: str | None = ...,
        gch_location: str | None = ...,
        gch_keyring: str | None = ...,
        gch_cryptokey: str | None = ...,
        gch_cryptokey_version: str | None = ...,
        gch_cloud_service_name: str | None = ...,
        gch_cryptokey_algorithm: Literal["rsa-sign-pkcs1-2048-sha256", "rsa-sign-pkcs1-3072-sha256", "rsa-sign-pkcs1-4096-sha256", "rsa-sign-pkcs1-4096-sha512", "rsa-sign-pss-2048-sha256", "rsa-sign-pss-3072-sha256", "rsa-sign-pss-4096-sha256", "rsa-sign-pss-4096-sha512", "ec-sign-p256-sha256", "ec-sign-p384-sha384", "ec-sign-secp256k1-sha256"] | None = ...,
        details: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def put(
        self,
        payload_dict: HsmLocalPayload | None = ...,
        name: str | None = ...,
        comments: str | None = ...,
        vendor: Literal["unknown", "gch"] | None = ...,
        api_version: Literal["unknown", "gch-default"] | None = ...,
        certificate: str | None = ...,
        range: Literal["global", "vdom"] | None = ...,
        source: Literal["factory", "user", "bundle"] | None = ...,
        gch_url: str | None = ...,
        gch_project: str | None = ...,
        gch_location: str | None = ...,
        gch_keyring: str | None = ...,
        gch_cryptokey: str | None = ...,
        gch_cryptokey_version: str | None = ...,
        gch_cloud_service_name: str | None = ...,
        gch_cryptokey_algorithm: Literal["rsa-sign-pkcs1-2048-sha256", "rsa-sign-pkcs1-3072-sha256", "rsa-sign-pkcs1-4096-sha256", "rsa-sign-pkcs1-4096-sha512", "rsa-sign-pss-2048-sha256", "rsa-sign-pss-3072-sha256", "rsa-sign-pss-4096-sha256", "rsa-sign-pss-4096-sha512", "ec-sign-p256-sha256", "ec-sign-p384-sha384", "ec-sign-secp256k1-sha256"] | None = ...,
        details: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> Union[bool, Coroutine[Any, Any, bool]]: ...
    
    def set(
        self,
        payload_dict: HsmLocalPayload | None = ...,
        vdom: str | bool | None = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> Union[list[str], list[dict[str, Any]]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> dict[str, Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> dict[str, Any]: ...
    
    @staticmethod
    def schema() -> dict[str, Any]: ...


__all__ = [
    "HsmLocal",
    "HsmLocalPayload",
]


from typing import TypedDict, Literal, NotRequired, Any, Coroutine, Union

# Payload TypedDict for IDE autocomplete
class InterControllerPayload(TypedDict, total=False):
    """
    Type hints for wireless_controller/inter_controller payload fields.
    
    Use this for IDE autocomplete when building payload dicts:
        payload: InterControllerPayload = {
            "field": "value",  # <- autocomplete shows all fields
        }
    """
    inter_controller_mode: NotRequired[Literal["disable", "l2-roaming", "1+1"]]  # Configure inter-controller mode (disable, l2-roaming, 1+1, d
    l3_roaming: NotRequired[Literal["enable", "disable"]]  # Enable/disable layer 3 roaming (default = disable).
    inter_controller_key: NotRequired[str]  # Secret key for inter-controller communications.
    inter_controller_pri: NotRequired[Literal["primary", "secondary"]]  # Configure inter-controller's priority (primary or secondary,
    fast_failover_max: NotRequired[int]  # Maximum number of retransmissions for fast failover HA messa
    fast_failover_wait: NotRequired[int]  # Minimum wait time before an AP transitions from secondary co
    inter_controller_peer: NotRequired[list[dict[str, Any]]]  # Fast failover peer wireless controller list.


class InterController:
    """
    Configure inter wireless controller operation.
    
    Path: wireless_controller/inter_controller
    Category: cmdb
    """
    
    def get(
        self,
        name: str | None = ...,
        filter: str | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def post(
        self,
        payload_dict: InterControllerPayload | None = ...,
        inter_controller_mode: Literal["disable", "l2-roaming", "1+1"] | None = ...,
        l3_roaming: Literal["enable", "disable"] | None = ...,
        inter_controller_key: str | None = ...,
        inter_controller_pri: Literal["primary", "secondary"] | None = ...,
        fast_failover_max: int | None = ...,
        fast_failover_wait: int | None = ...,
        inter_controller_peer: list[dict[str, Any]] | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def put(
        self,
        payload_dict: InterControllerPayload | None = ...,
        inter_controller_mode: Literal["disable", "l2-roaming", "1+1"] | None = ...,
        l3_roaming: Literal["enable", "disable"] | None = ...,
        inter_controller_key: str | None = ...,
        inter_controller_pri: Literal["primary", "secondary"] | None = ...,
        fast_failover_max: int | None = ...,
        fast_failover_wait: int | None = ...,
        inter_controller_peer: list[dict[str, Any]] | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> Union[bool, Coroutine[Any, Any, bool]]: ...
    
    def set(
        self,
        payload_dict: InterControllerPayload | None = ...,
        vdom: str | bool | None = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> Union[list[str], list[dict[str, Any]]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> dict[str, Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> dict[str, Any]: ...
    
    @staticmethod
    def schema() -> dict[str, Any]: ...


__all__ = [
    "InterController",
    "InterControllerPayload",
]


from typing import TypedDict, Literal, NotRequired, Any, Coroutine, Union

# Payload TypedDict for IDE autocomplete
class QosProfilePayload(TypedDict, total=False):
    """
    Type hints for wireless_controller/qos_profile payload fields.
    
    Use this for IDE autocomplete when building payload dicts:
        payload: QosProfilePayload = {
            "field": "value",  # <- autocomplete shows all fields
        }
    """
    name: NotRequired[str]  # WiFi QoS profile name.
    comment: NotRequired[str]  # Comment.
    uplink: NotRequired[int]  # Maximum uplink bandwidth for Virtual Access Points (VAPs) (0
    downlink: NotRequired[int]  # Maximum downlink bandwidth for Virtual Access Points (VAPs) 
    uplink_sta: NotRequired[int]  # Maximum uplink bandwidth for clients (0 - 2097152 Kbps, defa
    downlink_sta: NotRequired[int]  # Maximum downlink bandwidth for clients (0 - 2097152 Kbps, de
    burst: NotRequired[Literal["enable", "disable"]]  # Enable/disable client rate burst.
    wmm: NotRequired[Literal["enable", "disable"]]  # Enable/disable WiFi multi-media (WMM) control.
    wmm_uapsd: NotRequired[Literal["enable", "disable"]]  # Enable/disable WMM Unscheduled Automatic Power Save Delivery
    call_admission_control: NotRequired[Literal["enable", "disable"]]  # Enable/disable WMM call admission control.
    call_capacity: NotRequired[int]  # Maximum number of Voice over WLAN (VoWLAN) phones allowed (0
    bandwidth_admission_control: NotRequired[Literal["enable", "disable"]]  # Enable/disable WMM bandwidth admission control.
    bandwidth_capacity: NotRequired[int]  # Maximum bandwidth capacity allowed (1 - 600000 Kbps, default
    dscp_wmm_mapping: NotRequired[Literal["enable", "disable"]]  # Enable/disable Differentiated Services Code Point (DSCP) map
    dscp_wmm_vo: NotRequired[list[dict[str, Any]]]  # DSCP mapping for voice access (default = 48 56).
    dscp_wmm_vi: NotRequired[list[dict[str, Any]]]  # DSCP mapping for video access (default = 32 40).
    dscp_wmm_be: NotRequired[list[dict[str, Any]]]  # DSCP mapping for best effort access (default = 0 24).
    dscp_wmm_bk: NotRequired[list[dict[str, Any]]]  # DSCP mapping for background access (default = 8 16).
    wmm_dscp_marking: NotRequired[Literal["enable", "disable"]]  # Enable/disable WMM Differentiated Services Code Point (DSCP)
    wmm_vo_dscp: NotRequired[int]  # DSCP marking for voice access (default = 48).
    wmm_vi_dscp: NotRequired[int]  # DSCP marking for video access (default = 32).
    wmm_be_dscp: NotRequired[int]  # DSCP marking for best effort access (default = 0).
    wmm_bk_dscp: NotRequired[int]  # DSCP marking for background access (default = 8).


class QosProfile:
    """
    Configure WiFi quality of service (QoS) profiles.
    
    Path: wireless_controller/qos_profile
    Category: cmdb
    Primary Key: name
    """
    
    def get(
        self,
        name: str | None = ...,
        filter: str | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def post(
        self,
        payload_dict: QosProfilePayload | None = ...,
        name: str | None = ...,
        comment: str | None = ...,
        uplink: int | None = ...,
        downlink: int | None = ...,
        uplink_sta: int | None = ...,
        downlink_sta: int | None = ...,
        burst: Literal["enable", "disable"] | None = ...,
        wmm: Literal["enable", "disable"] | None = ...,
        wmm_uapsd: Literal["enable", "disable"] | None = ...,
        call_admission_control: Literal["enable", "disable"] | None = ...,
        call_capacity: int | None = ...,
        bandwidth_admission_control: Literal["enable", "disable"] | None = ...,
        bandwidth_capacity: int | None = ...,
        dscp_wmm_mapping: Literal["enable", "disable"] | None = ...,
        dscp_wmm_vo: list[dict[str, Any]] | None = ...,
        dscp_wmm_vi: list[dict[str, Any]] | None = ...,
        dscp_wmm_be: list[dict[str, Any]] | None = ...,
        dscp_wmm_bk: list[dict[str, Any]] | None = ...,
        wmm_dscp_marking: Literal["enable", "disable"] | None = ...,
        wmm_vo_dscp: int | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def put(
        self,
        payload_dict: QosProfilePayload | None = ...,
        name: str | None = ...,
        comment: str | None = ...,
        uplink: int | None = ...,
        downlink: int | None = ...,
        uplink_sta: int | None = ...,
        downlink_sta: int | None = ...,
        burst: Literal["enable", "disable"] | None = ...,
        wmm: Literal["enable", "disable"] | None = ...,
        wmm_uapsd: Literal["enable", "disable"] | None = ...,
        call_admission_control: Literal["enable", "disable"] | None = ...,
        call_capacity: int | None = ...,
        bandwidth_admission_control: Literal["enable", "disable"] | None = ...,
        bandwidth_capacity: int | None = ...,
        dscp_wmm_mapping: Literal["enable", "disable"] | None = ...,
        dscp_wmm_vo: list[dict[str, Any]] | None = ...,
        dscp_wmm_vi: list[dict[str, Any]] | None = ...,
        dscp_wmm_be: list[dict[str, Any]] | None = ...,
        dscp_wmm_bk: list[dict[str, Any]] | None = ...,
        wmm_dscp_marking: Literal["enable", "disable"] | None = ...,
        wmm_vo_dscp: int | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> Union[bool, Coroutine[Any, Any, bool]]: ...
    
    def set(
        self,
        payload_dict: QosProfilePayload | None = ...,
        vdom: str | bool | None = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> Union[list[str], list[dict[str, Any]]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> dict[str, Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> dict[str, Any]: ...
    
    @staticmethod
    def schema() -> dict[str, Any]: ...


__all__ = [
    "QosProfile",
    "QosProfilePayload",
]


from typing import TypedDict, Literal, NotRequired, Any, Coroutine, Union

# Payload TypedDict for IDE autocomplete
class PolicyPayload(TypedDict, total=False):
    """
    Type hints for firewall/policy payload fields.
    
    Use this for IDE autocomplete when building payload dicts:
        payload: PolicyPayload = {
            "field": "value",  # <- autocomplete shows all fields
        }
    """
    policyid: NotRequired[int]  # Policy ID (0 - 4294967294).
    status: NotRequired[Literal["enable", "disable"]]  # Enable or disable this policy.
    name: NotRequired[str]  # Policy name.
    uuid: NotRequired[str]  # Universally Unique Identifier (UUID; automatically assigned 
    srcintf: list[dict[str, Any]]  # Incoming (ingress) interface.
    dstintf: list[dict[str, Any]]  # Outgoing (egress) interface.
    action: NotRequired[Literal["accept", "deny", "ipsec"]]  # Policy action (accept/deny/ipsec).
    nat64: NotRequired[Literal["enable", "disable"]]  # Enable/disable NAT64.
    nat46: NotRequired[Literal["enable", "disable"]]  # Enable/disable NAT46.
    ztna_status: NotRequired[Literal["enable", "disable"]]  # Enable/disable zero trust access.
    ztna_device_ownership: NotRequired[Literal["enable", "disable"]]  # Enable/disable zero trust device ownership.
    srcaddr: NotRequired[list[dict[str, Any]]]  # Source IPv4 address and address group names.
    dstaddr: NotRequired[list[dict[str, Any]]]  # Destination IPv4 address and address group names.
    srcaddr6: NotRequired[list[dict[str, Any]]]  # Source IPv6 address name and address group names.
    dstaddr6: NotRequired[list[dict[str, Any]]]  # Destination IPv6 address name and address group names.
    ztna_ems_tag: NotRequired[list[dict[str, Any]]]  # Source ztna-ems-tag names.
    ztna_ems_tag_secondary: NotRequired[list[dict[str, Any]]]  # Source ztna-ems-tag-secondary names.
    ztna_tags_match_logic: NotRequired[Literal["or", "and"]]  # ZTNA tag matching logic.
    ztna_geo_tag: NotRequired[list[dict[str, Any]]]  # Source ztna-geo-tag names.
    internet_service: NotRequired[Literal["enable", "disable"]]  # Enable/disable use of Internet Services for this policy. If 
    internet_service_name: NotRequired[list[dict[str, Any]]]  # Internet Service name.
    internet_service_group: NotRequired[list[dict[str, Any]]]  # Internet Service group name.
    internet_service_custom: NotRequired[list[dict[str, Any]]]  # Custom Internet Service name.
    network_service_dynamic: NotRequired[list[dict[str, Any]]]  # Dynamic Network Service name.
    internet_service_custom_group: NotRequired[list[dict[str, Any]]]  # Custom Internet Service group name.
    internet_service_src: NotRequired[Literal["enable", "disable"]]  # Enable/disable use of Internet Services in source for this p
    internet_service_src_name: NotRequired[list[dict[str, Any]]]  # Internet Service source name.
    internet_service_src_group: NotRequired[list[dict[str, Any]]]  # Internet Service source group name.
    internet_service_src_custom: NotRequired[list[dict[str, Any]]]  # Custom Internet Service source name.
    network_service_src_dynamic: NotRequired[list[dict[str, Any]]]  # Dynamic Network Service source name.
    internet_service_src_custom_group: NotRequired[list[dict[str, Any]]]  # Custom Internet Service source group name.
    reputation_minimum: NotRequired[int]  # Minimum Reputation to take action.
    reputation_direction: NotRequired[Literal["source", "destination"]]  # Direction of the initial traffic for reputation to take effe
    src_vendor_mac: NotRequired[list[dict[str, Any]]]  # Vendor MAC source ID.
    internet_service6: NotRequired[Literal["enable", "disable"]]  # Enable/disable use of IPv6 Internet Services for this policy
    internet_service6_name: NotRequired[list[dict[str, Any]]]  # IPv6 Internet Service name.
    internet_service6_group: NotRequired[list[dict[str, Any]]]  # Internet Service group name.
    internet_service6_custom: NotRequired[list[dict[str, Any]]]  # Custom IPv6 Internet Service name.
    internet_service6_custom_group: NotRequired[list[dict[str, Any]]]  # Custom Internet Service6 group name.
    internet_service6_src: NotRequired[Literal["enable", "disable"]]  # Enable/disable use of IPv6 Internet Services in source for t
    internet_service6_src_name: NotRequired[list[dict[str, Any]]]  # IPv6 Internet Service source name.
    internet_service6_src_group: NotRequired[list[dict[str, Any]]]  # Internet Service6 source group name.
    internet_service6_src_custom: NotRequired[list[dict[str, Any]]]  # Custom IPv6 Internet Service source name.
    internet_service6_src_custom_group: NotRequired[list[dict[str, Any]]]  # Custom Internet Service6 source group name.
    reputation_minimum6: NotRequired[int]  # IPv6 Minimum Reputation to take action.
    reputation_direction6: NotRequired[Literal["source", "destination"]]  # Direction of the initial traffic for IPv6 reputation to take
    rtp_nat: NotRequired[Literal["disable", "enable"]]  # Enable Real Time Protocol (RTP) NAT.
    rtp_addr: list[dict[str, Any]]  # Address names if this is an RTP NAT policy.
    send_deny_packet: NotRequired[Literal["disable", "enable"]]  # Enable to send a reply when a session is denied or blocked b
    firewall_session_dirty: NotRequired[Literal["check-all", "check-new"]]  # How to handle sessions if the configuration of this firewall
    schedule: str  # Schedule name.
    schedule_timeout: NotRequired[Literal["enable", "disable"]]  # Enable to force current sessions to end when the schedule ob
    policy_expiry: NotRequired[Literal["enable", "disable"]]  # Enable/disable policy expiry.
    policy_expiry_date: NotRequired[str]  # Policy expiry date (YYYY-MM-DD HH:MM:SS).
    policy_expiry_date_utc: NotRequired[str]  # Policy expiry date and time, in epoch format.
    service: NotRequired[list[dict[str, Any]]]  # Service and service group names.
    tos_mask: NotRequired[str]  # Non-zero bit positions are used for comparison while zero bi
    tos: NotRequired[str]  # ToS (Type of Service) value used for comparison.
    tos_negate: NotRequired[Literal["enable", "disable"]]  # Enable negated TOS match.
    anti_replay: NotRequired[Literal["enable", "disable"]]  # Enable/disable anti-replay check.
    tcp_session_without_syn: NotRequired[Literal["all", "data-only", "disable"]]  # Enable/disable creation of TCP session without SYN flag.
    geoip_anycast: NotRequired[Literal["enable", "disable"]]  # Enable/disable recognition of anycast IP addresses using the
    geoip_match: NotRequired[Literal["physical-location", "registered-location"]]  # Match geography address based either on its physical locatio
    dynamic_shaping: NotRequired[Literal["enable", "disable"]]  # Enable/disable dynamic RADIUS defined traffic shaping.
    passive_wan_health_measurement: NotRequired[Literal["enable", "disable"]]  # Enable/disable passive WAN health measurement. When enabled,
    app_monitor: NotRequired[Literal["enable", "disable"]]  # Enable/disable application TCP metrics in session logs.When 
    utm_status: NotRequired[Literal["enable", "disable"]]  # Enable to add one or more security profiles (AV, IPS, etc.) 
    inspection_mode: NotRequired[Literal["proxy", "flow"]]  # Policy inspection mode (Flow/proxy). Default is Flow mode.
    http_policy_redirect: NotRequired[Literal["enable", "disable", "legacy"]]  # Redirect HTTP(S) traffic to matching transparent web proxy p
    ssh_policy_redirect: NotRequired[Literal["enable", "disable"]]  # Redirect SSH traffic to matching transparent proxy policy.
    ztna_policy_redirect: NotRequired[Literal["enable", "disable"]]  # Redirect ZTNA traffic to matching Access-Proxy proxy-policy.
    webproxy_profile: NotRequired[str]  # Webproxy profile name.
    profile_type: NotRequired[Literal["single", "group"]]  # Determine whether the firewall policy allows security profil
    profile_group: NotRequired[str]  # Name of profile group.
    profile_protocol_options: NotRequired[str]  # Name of an existing Protocol options profile.
    ssl_ssh_profile: NotRequired[str]  # Name of an existing SSL SSH profile.
    av_profile: NotRequired[str]  # Name of an existing Antivirus profile.
    webfilter_profile: NotRequired[str]  # Name of an existing Web filter profile.
    dnsfilter_profile: NotRequired[str]  # Name of an existing DNS filter profile.
    emailfilter_profile: NotRequired[str]  # Name of an existing email filter profile.
    dlp_profile: NotRequired[str]  # Name of an existing DLP profile.
    file_filter_profile: NotRequired[str]  # Name of an existing file-filter profile.
    ips_sensor: NotRequired[str]  # Name of an existing IPS sensor.
    application_list: NotRequired[str]  # Name of an existing Application list.
    voip_profile: NotRequired[str]  # Name of an existing VoIP (voipd) profile.
    ips_voip_filter: NotRequired[str]  # Name of an existing VoIP (ips) profile.
    sctp_filter_profile: NotRequired[str]  # Name of an existing SCTP filter profile.
    diameter_filter_profile: NotRequired[str]  # Name of an existing Diameter filter profile.
    virtual_patch_profile: NotRequired[str]  # Name of an existing virtual-patch profile.
    icap_profile: NotRequired[str]  # Name of an existing ICAP profile.
    videofilter_profile: NotRequired[str]  # Name of an existing VideoFilter profile.
    waf_profile: NotRequired[str]  # Name of an existing Web application firewall profile.
    ssh_filter_profile: NotRequired[str]  # Name of an existing SSH filter profile.
    casb_profile: NotRequired[str]  # Name of an existing CASB profile.
    logtraffic: NotRequired[Literal["all", "utm", "disable"]]  # Enable or disable logging. Log all sessions or security prof
    logtraffic_start: NotRequired[Literal["enable", "disable"]]  # Record logs when a session starts.
    log_http_transaction: NotRequired[Literal["enable", "disable"]]  # Enable/disable HTTP transaction log.
    capture_packet: NotRequired[Literal["enable", "disable"]]  # Enable/disable capture packets.
    auto_asic_offload: NotRequired[Literal["enable", "disable"]]  # Enable/disable policy traffic ASIC offloading.
    wanopt: NotRequired[Literal["enable", "disable"]]  # Enable/disable WAN optimization.
    wanopt_detection: NotRequired[Literal["active", "passive", "off"]]  # WAN optimization auto-detection mode.
    wanopt_passive_opt: NotRequired[Literal["default", "transparent", "non-transparent"]]  # WAN optimization passive mode options. This option decides w
    wanopt_profile: str  # WAN optimization profile.
    wanopt_peer: str  # WAN optimization peer.
    webcache: NotRequired[Literal["enable", "disable"]]  # Enable/disable web cache.
    webcache_https: NotRequired[Literal["disable", "enable"]]  # Enable/disable web cache for HTTPS.
    webproxy_forward_server: NotRequired[str]  # Webproxy forward server name.
    traffic_shaper: NotRequired[str]  # Traffic shaper.
    traffic_shaper_reverse: NotRequired[str]  # Reverse traffic shaper.
    per_ip_shaper: NotRequired[str]  # Per-IP traffic shaper.
    nat: NotRequired[Literal["enable", "disable"]]  # Enable/disable source NAT.
    pcp_outbound: NotRequired[Literal["enable", "disable"]]  # Enable/disable PCP outbound SNAT.
    pcp_inbound: NotRequired[Literal["enable", "disable"]]  # Enable/disable PCP inbound DNAT.
    pcp_poolname: NotRequired[list[dict[str, Any]]]  # PCP pool names.
    permit_any_host: NotRequired[Literal["enable", "disable"]]  # Accept UDP packets from any host.
    permit_stun_host: NotRequired[Literal["enable", "disable"]]  # Accept UDP packets from any Session Traversal Utilities for 
    fixedport: NotRequired[Literal["enable", "disable"]]  # Enable to prevent source NAT from changing a session's sourc
    port_preserve: NotRequired[Literal["enable", "disable"]]  # Enable/disable preservation of the original source port from
    port_random: NotRequired[Literal["enable", "disable"]]  # Enable/disable random source port selection for source NAT.
    ippool: NotRequired[Literal["enable", "disable"]]  # Enable to use IP Pools for source NAT.
    poolname: NotRequired[list[dict[str, Any]]]  # IP Pool names.
    poolname6: NotRequired[list[dict[str, Any]]]  # IPv6 pool names.
    session_ttl: NotRequired[str]  # TTL in seconds for sessions accepted by this policy (0 means
    vlan_cos_fwd: NotRequired[int]  # VLAN forward direction user priority: 255 passthrough, 0 low
    vlan_cos_rev: NotRequired[int]  # VLAN reverse direction user priority: 255 passthrough, 0 low
    inbound: NotRequired[Literal["enable", "disable"]]  # Policy-based IPsec VPN: only traffic from the remote network
    outbound: NotRequired[Literal["enable", "disable"]]  # Policy-based IPsec VPN: only traffic from the internal netwo
    natinbound: NotRequired[Literal["enable", "disable"]]  # Policy-based IPsec VPN: apply destination NAT to inbound tra
    natoutbound: NotRequired[Literal["enable", "disable"]]  # Policy-based IPsec VPN: apply source NAT to outbound traffic
    fec: NotRequired[Literal["enable", "disable"]]  # Enable/disable Forward Error Correction on traffic matching 
    wccp: NotRequired[Literal["enable", "disable"]]  # Enable/disable forwarding traffic matching this policy to a 
    ntlm: NotRequired[Literal["enable", "disable"]]  # Enable/disable NTLM authentication.
    ntlm_guest: NotRequired[Literal["enable", "disable"]]  # Enable/disable NTLM guest user access.
    ntlm_enabled_browsers: NotRequired[list[dict[str, Any]]]  # HTTP-User-Agent value of supported browsers.
    fsso_agent_for_ntlm: NotRequired[str]  # FSSO agent to use for NTLM authentication.
    groups: NotRequired[list[dict[str, Any]]]  # Names of user groups that can authenticate with this policy.
    users: NotRequired[list[dict[str, Any]]]  # Names of individual users that can authenticate with this po
    fsso_groups: NotRequired[list[dict[str, Any]]]  # Names of FSSO groups.
    scim: NotRequired[Literal["enable", "disable"]]  # Enable/disable SCIM (default = disable).
    saml_server: NotRequired[str]  # SAML server name.
    scim_users: NotRequired[list[dict[str, Any]]]  # Names of SCIM users.
    scim_groups: NotRequired[list[dict[str, Any]]]  # Names of SCIM groups.
    auth_path: NotRequired[Literal["enable", "disable"]]  # Enable/disable authentication-based routing.
    disclaimer: NotRequired[Literal["enable", "disable"]]  # Enable/disable user authentication disclaimer.
    email_collect: NotRequired[Literal["enable", "disable"]]  # Enable/disable email collection.
    vpntunnel: str  # Policy-based IPsec VPN: name of the IPsec VPN Phase 1.
    natip: NotRequired[str]  # Policy-based IPsec VPN: source NAT IP address for outgoing t
    match_vip: NotRequired[Literal["enable", "disable"]]  # Enable to match packets that have had their destination addr
    match_vip_only: NotRequired[Literal["enable", "disable"]]  # Enable/disable matching of only those packets that have had 
    diffserv_copy: NotRequired[Literal["enable", "disable"]]  # Enable to copy packet's DiffServ values from session's origi
    diffserv_forward: NotRequired[Literal["enable", "disable"]]  # Enable to change packet's DiffServ values to the specified d
    diffserv_reverse: NotRequired[Literal["enable", "disable"]]  # Enable to change packet's reverse (reply) DiffServ values to
    diffservcode_forward: NotRequired[str]  # Change packet's DiffServ to this value.
    diffservcode_rev: NotRequired[str]  # Change packet's reverse (reply) DiffServ to this value.
    tcp_mss_sender: NotRequired[int]  # Sender TCP maximum segment size (MSS).
    tcp_mss_receiver: NotRequired[int]  # Receiver TCP maximum segment size (MSS).
    comments: NotRequired[str]  # Comment.
    auth_cert: NotRequired[str]  # HTTPS server certificate for policy authentication.
    auth_redirect_addr: NotRequired[str]  # HTTP-to-HTTPS redirect address for firewall authentication.
    redirect_url: NotRequired[str]  # URL users are directed to after seeing and accepting the dis
    identity_based_route: NotRequired[str]  # Name of identity-based routing rule.
    block_notification: NotRequired[Literal["enable", "disable"]]  # Enable/disable block notification.
    custom_log_fields: NotRequired[list[dict[str, Any]]]  # Custom fields to append to log messages for this policy.
    replacemsg_override_group: NotRequired[str]  # Override the default replacement message group for this poli
    srcaddr_negate: NotRequired[Literal["enable", "disable"]]  # When enabled srcaddr specifies what the source address must 
    srcaddr6_negate: NotRequired[Literal["enable", "disable"]]  # When enabled srcaddr6 specifies what the source address must
    dstaddr_negate: NotRequired[Literal["enable", "disable"]]  # When enabled dstaddr specifies what the destination address 
    dstaddr6_negate: NotRequired[Literal["enable", "disable"]]  # When enabled dstaddr6 specifies what the destination address
    ztna_ems_tag_negate: NotRequired[Literal["enable", "disable"]]  # When enabled ztna-ems-tag specifies what the tags must NOT b
    service_negate: NotRequired[Literal["enable", "disable"]]  # When enabled service specifies what the service must NOT be.
    internet_service_negate: NotRequired[Literal["enable", "disable"]]  # When enabled internet-service specifies what the service mus
    internet_service_src_negate: NotRequired[Literal["enable", "disable"]]  # When enabled internet-service-src specifies what the service
    internet_service6_negate: NotRequired[Literal["enable", "disable"]]  # When enabled internet-service6 specifies what the service mu
    internet_service6_src_negate: NotRequired[Literal["enable", "disable"]]  # When enabled internet-service6-src specifies what the servic
    timeout_send_rst: NotRequired[Literal["enable", "disable"]]  # Enable/disable sending RST packets when TCP sessions expire.
    captive_portal_exempt: NotRequired[Literal["enable", "disable"]]  # Enable to exempt some users from the captive portal.
    decrypted_traffic_mirror: NotRequired[str]  # Decrypted traffic mirror.
    dsri: NotRequired[Literal["enable", "disable"]]  # Enable DSRI to ignore HTTP server responses.
    radius_mac_auth_bypass: NotRequired[Literal["enable", "disable"]]  # Enable MAC authentication bypass. The bypassed MAC address m
    radius_ip_auth_bypass: NotRequired[Literal["enable", "disable"]]  # Enable IP authentication bypass. The bypassed IP address mus
    delay_tcp_npu_session: NotRequired[Literal["enable", "disable"]]  # Enable TCP NPU session delay to guarantee packet order of 3-
    vlan_filter: NotRequired[str]  # VLAN ranges to allow
    sgt_check: NotRequired[Literal["enable", "disable"]]  # Enable/disable security group tags (SGT) check.
    sgt: NotRequired[list[dict[str, Any]]]  # Security group tags.
    internet_service_fortiguard: NotRequired[list[dict[str, Any]]]  # FortiGuard Internet Service name.
    internet_service_src_fortiguard: NotRequired[list[dict[str, Any]]]  # FortiGuard Internet Service source name.
    internet_service6_fortiguard: NotRequired[list[dict[str, Any]]]  # FortiGuard IPv6 Internet Service name.
    internet_service6_src_fortiguard: NotRequired[list[dict[str, Any]]]  # FortiGuard IPv6 Internet Service source name.


class Policy:
    """
    Configure IPv4/IPv6 policies.
    
    Path: firewall/policy
    Category: cmdb
    Primary Key: policyid
    """
    
    def get(
        self,
        policyid: int | None = ...,
        filter: str | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def post(
        self,
        payload_dict: PolicyPayload | None = ...,
        policyid: int | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        name: str | None = ...,
        uuid: str | None = ...,
        srcintf: list[dict[str, Any]] | None = ...,
        dstintf: list[dict[str, Any]] | None = ...,
        action: Literal["accept", "deny", "ipsec"] | None = ...,
        nat64: Literal["enable", "disable"] | None = ...,
        nat46: Literal["enable", "disable"] | None = ...,
        ztna_status: Literal["enable", "disable"] | None = ...,
        ztna_device_ownership: Literal["enable", "disable"] | None = ...,
        srcaddr: list[dict[str, Any]] | None = ...,
        dstaddr: list[dict[str, Any]] | None = ...,
        srcaddr6: list[dict[str, Any]] | None = ...,
        dstaddr6: list[dict[str, Any]] | None = ...,
        ztna_ems_tag: list[dict[str, Any]] | None = ...,
        ztna_ems_tag_secondary: list[dict[str, Any]] | None = ...,
        ztna_tags_match_logic: Literal["or", "and"] | None = ...,
        ztna_geo_tag: list[dict[str, Any]] | None = ...,
        internet_service: Literal["enable", "disable"] | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def put(
        self,
        payload_dict: PolicyPayload | None = ...,
        policyid: int | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        name: str | None = ...,
        uuid: str | None = ...,
        srcintf: list[dict[str, Any]] | None = ...,
        dstintf: list[dict[str, Any]] | None = ...,
        action: Literal["accept", "deny", "ipsec"] | None = ...,
        nat64: Literal["enable", "disable"] | None = ...,
        nat46: Literal["enable", "disable"] | None = ...,
        ztna_status: Literal["enable", "disable"] | None = ...,
        ztna_device_ownership: Literal["enable", "disable"] | None = ...,
        srcaddr: list[dict[str, Any]] | None = ...,
        dstaddr: list[dict[str, Any]] | None = ...,
        srcaddr6: list[dict[str, Any]] | None = ...,
        dstaddr6: list[dict[str, Any]] | None = ...,
        ztna_ems_tag: list[dict[str, Any]] | None = ...,
        ztna_ems_tag_secondary: list[dict[str, Any]] | None = ...,
        ztna_tags_match_logic: Literal["or", "and"] | None = ...,
        ztna_geo_tag: list[dict[str, Any]] | None = ...,
        internet_service: Literal["enable", "disable"] | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def delete(
        self,
        policyid: int | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def exists(
        self,
        policyid: int,
        vdom: str | bool | None = ...,
    ) -> Union[bool, Coroutine[Any, Any, bool]]: ...
    
    def set(
        self,
        payload_dict: PolicyPayload | None = ...,
        vdom: str | bool | None = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> Union[list[str], list[dict[str, Any]]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> dict[str, Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> dict[str, Any]: ...
    
    @staticmethod
    def schema() -> dict[str, Any]: ...


__all__ = [
    "Policy",
    "PolicyPayload",
]


"""
Validation helpers for user/radius endpoint.

Each endpoint has its own validation file to keep validation logic
separate and maintainable. Use central cmdb._helpers tools for common tasks.

Auto-generated from OpenAPI specification
Customize as needed for endpoint-specific business logic.
"""

from typing import Any, TypedDict, NotRequired, Literal

# Import common validators from central _helpers module
from hfortix_fortios._helpers import (
    validate_enable_disable,
    validate_integer_range,
    validate_string_length,
    validate_port_number,
    validate_ip_address,
    validate_ipv6_address,
    validate_mac_address,
)

# ============================================================================
# Required Fields Validation
# Auto-generated from schema
# ============================================================================

# ⚠️  IMPORTANT: FortiOS schemas have known issues with required field marking:
#
# 1. FALSE POSITIVES: Some fields marked "required" have default values,
#    meaning they're optional (filtered out by generator)
#
# 2. CONDITIONAL REQUIREMENTS: Many endpoints require EITHER field A OR field B:
#    - firewall.policy: requires (srcaddr + dstaddr) OR (srcaddr6 + dstaddr6)
#    - These conditional requirements cannot be expressed in a simple list
#
# 3. SPECIALIZED FEATURES: Fields for WAN optimization, VPN, NAT64, etc.
#    are marked "required" but only apply when using those features
#
# The REQUIRED_FIELDS list below is INFORMATIONAL ONLY and shows fields that:
# - Are marked required in the schema
# - Don't have non-empty default values
# - Aren't specialized feature fields
#
# Do NOT use this list for strict validation - test with the actual FortiOS API!

# Fields marked as required (after filtering false positives)
REQUIRED_FIELDS = [
    "server",  # Primary RADIUS server CN domain name or IP address.
    "secret",  # Pre-shared secret key used to access the primary RADIUS server.
    "interface",  # Specify outgoing interface to reach server.
    "rsso-secret",  # RADIUS secret used by the RADIUS accounting server.
    "rsso-endpoint-block-attribute",  # RADIUS attributes used to block a user.
]

# Fields with defaults (optional)
FIELDS_WITH_DEFAULTS = {
    "name": "",
    "server": "",
    "secondary-server": "",
    "tertiary-server": "",
    "timeout": 5,
    "status-ttl": 300,
    "all-usergroup": "disable",
    "use-management-vdom": "disable",
    "switch-controller-nas-ip-dynamic": "disable",
    "nas-ip": "0.0.0.0",
    "nas-id-type": "legacy",
    "call-station-id-type": "legacy",
    "nas-id": "",
    "acct-interim-interval": 0,
    "radius-coa": "disable",
    "radius-port": 0,
    "h3c-compatibility": "disable",
    "auth-type": "auto",
    "source-ip": "",
    "source-ip-interface": "",
    "username-case-sensitive": "disable",
    "group-override-attr-type": "",
    "password-renewal": "enable",
    "require-message-authenticator": "enable",
    "password-encoding": "auto",
    "mac-username-delimiter": "hyphen",
    "mac-password-delimiter": "hyphen",
    "mac-case": "lowercase",
    "acct-all-servers": "disable",
    "switch-controller-acct-fast-framedip-detect": 2,
    "interface-select-method": "auto",
    "interface": "",
    "vrf-select": 0,
    "switch-controller-service-type": "",
    "transport-protocol": "udp",
    "tls-min-proto-version": "default",
    "ca-cert": "",
    "client-cert": "",
    "server-identity-check": "enable",
    "account-key-processing": "same",
    "account-key-cert-field": "othername",
    "rsso": "disable",
    "rsso-radius-server-port": 1813,
    "rsso-radius-response": "disable",
    "rsso-validate-request-secret": "disable",
    "rsso-endpoint-attribute": "Calling-Station-Id",
    "rsso-endpoint-block-attribute": "",
    "sso-attribute": "Class",
    "sso-attribute-key": "",
    "sso-attribute-value-override": "enable",
    "rsso-context-timeout": 28800,
    "rsso-log-period": 0,
    "rsso-log-flags": "protocol-error profile-missing accounting-stop-missed accounting-event endpoint-block radiusd-other",
    "rsso-flush-ip-session": "disable",
    "rsso-ep-one-ip-only": "disable",
    "delimiter": "plus",
}

# ============================================================================
# Deprecated Fields
# Auto-generated from schema - warns users about deprecated fields
# ============================================================================

# Deprecated fields with migration guidance
DEPRECATED_FIELDS = {
}

# ============================================================================
# Field Metadata (Type Information & Descriptions)
# Auto-generated from schema - use for IDE autocomplete and documentation
# ============================================================================

# Field types mapping
FIELD_TYPES = {
    "name": "string",  # RADIUS server entry name.
    "server": "string",  # Primary RADIUS server CN domain name or IP address.
    "secret": "password",  # Pre-shared secret key used to access the primary RADIUS serv
    "secondary-server": "string",  # Secondary RADIUS CN domain name or IP address.
    "secondary-secret": "password",  # Secret key to access the secondary server.
    "tertiary-server": "string",  # Tertiary RADIUS CN domain name or IP address.
    "tertiary-secret": "password",  # Secret key to access the tertiary server.
    "timeout": "integer",  # Time in seconds to retry connecting server.
    "status-ttl": "integer",  # Time for which server reachability is cached so that when a 
    "all-usergroup": "option",  # Enable/disable automatically including this RADIUS server in
    "use-management-vdom": "option",  # Enable/disable using management VDOM to send requests.
    "switch-controller-nas-ip-dynamic": "option",  # Enable/Disable switch-controller nas-ip dynamic to dynamical
    "nas-ip": "ipv4-address",  # IP address used to communicate with the RADIUS server and us
    "nas-id-type": "option",  # NAS identifier type configuration (default = legacy).
    "call-station-id-type": "option",  # Calling & Called station identifier type configuration (defa
    "nas-id": "string",  # Custom NAS identifier.
    "acct-interim-interval": "integer",  # Time in seconds between each accounting interim update messa
    "radius-coa": "option",  # Enable to allow a mechanism to change the attributes of an a
    "radius-port": "integer",  # RADIUS service port number.
    "h3c-compatibility": "option",  # Enable/disable compatibility with the H3C, a mechanism that 
    "auth-type": "option",  # Authentication methods/protocols permitted for this RADIUS s
    "source-ip": "string",  # Source IP address for communications to the RADIUS server.
    "source-ip-interface": "string",  # Source interface for communication with the RADIUS server.
    "username-case-sensitive": "option",  # Enable/disable case sensitive user names.
    "group-override-attr-type": "option",  # RADIUS attribute type to override user group information.
    "class": "string",  # Class attribute name(s).
    "password-renewal": "option",  # Enable/disable password renewal.
    "require-message-authenticator": "option",  # Require message authenticator in authentication response.
    "password-encoding": "option",  # Password encoding.
    "mac-username-delimiter": "option",  # MAC authentication username delimiter (default = hyphen).
    "mac-password-delimiter": "option",  # MAC authentication password delimiter (default = hyphen).
    "mac-case": "option",  # MAC authentication case (default = lowercase).
    "acct-all-servers": "option",  # Enable/disable sending of accounting messages to all configu
    "switch-controller-acct-fast-framedip-detect": "integer",  # Switch controller accounting message Framed-IP detection fro
    "interface-select-method": "option",  # Specify how to select outgoing interface to reach server.
    "interface": "string",  # Specify outgoing interface to reach server.
    "vrf-select": "integer",  # VRF ID used for connection to server.
    "switch-controller-service-type": "option",  # RADIUS service type.
    "transport-protocol": "option",  # Transport protocol to be used (default = udp).
    "tls-min-proto-version": "option",  # Minimum supported protocol version for TLS connections (defa
    "ca-cert": "string",  # CA of server to trust under TLS.
    "client-cert": "string",  # Client certificate to use under TLS.
    "server-identity-check": "option",  # Enable/disable RADIUS server identity check (verify server d
    "account-key-processing": "option",  # Account key processing operation. The FortiGate will keep ei
    "account-key-cert-field": "option",  # Define subject identity field in certificate for user access
    "rsso": "option",  # Enable/disable RADIUS based single sign on feature.
    "rsso-radius-server-port": "integer",  # UDP port to listen on for RADIUS Start and Stop records.
    "rsso-radius-response": "option",  # Enable/disable sending RADIUS response packets after receivi
    "rsso-validate-request-secret": "option",  # Enable/disable validating the RADIUS request shared secret i
    "rsso-secret": "password",  # RADIUS secret used by the RADIUS accounting server.
    "rsso-endpoint-attribute": "option",  # RADIUS attributes used to extract the user end point identif
    "rsso-endpoint-block-attribute": "option",  # RADIUS attributes used to block a user.
    "sso-attribute": "option",  # RADIUS attribute that contains the profile group name to be 
    "sso-attribute-key": "string",  # Key prefix for SSO group value in the SSO attribute.
    "sso-attribute-value-override": "option",  # Enable/disable override old attribute value with new value f
    "rsso-context-timeout": "integer",  # Time in seconds before the logged out user is removed from t
    "rsso-log-period": "integer",  # Time interval in seconds that group event log messages will 
    "rsso-log-flags": "option",  # Events to log.
    "rsso-flush-ip-session": "option",  # Enable/disable flushing user IP sessions on RADIUS accountin
    "rsso-ep-one-ip-only": "option",  # Enable/disable the replacement of old IP addresses with new 
    "delimiter": "option",  # Configure delimiter to be used for separating profile group 
    "accounting-server": "string",  # Additional accounting servers.
}

# Field descriptions (help text from FortiOS API)
FIELD_DESCRIPTIONS = {
    "name": "RADIUS server entry name.",
    "server": "Primary RADIUS server CN domain name or IP address.",
    "secret": "Pre-shared secret key used to access the primary RADIUS server.",
    "secondary-server": "Secondary RADIUS CN domain name or IP address.",
    "secondary-secret": "Secret key to access the secondary server.",
    "tertiary-server": "Tertiary RADIUS CN domain name or IP address.",
    "tertiary-secret": "Secret key to access the tertiary server.",
    "timeout": "Time in seconds to retry connecting server.",
    "status-ttl": "Time for which server reachability is cached so that when a server is unreachable, it will not be retried for at least this period of time (0 = cache disabled, default = 300).",
    "all-usergroup": "Enable/disable automatically including this RADIUS server in all user groups.",
    "use-management-vdom": "Enable/disable using management VDOM to send requests.",
    "switch-controller-nas-ip-dynamic": "Enable/Disable switch-controller nas-ip dynamic to dynamically set nas-ip.",
    "nas-ip": "IP address used to communicate with the RADIUS server and used as NAS-IP-Address and Called-Station-ID attributes.",
    "nas-id-type": "NAS identifier type configuration (default = legacy).",
    "call-station-id-type": "Calling & Called station identifier type configuration (default = legacy), this option is not available for 802.1x authentication. ",
    "nas-id": "Custom NAS identifier.",
    "acct-interim-interval": "Time in seconds between each accounting interim update message.",
    "radius-coa": "Enable to allow a mechanism to change the attributes of an authentication, authorization, and accounting session after it is authenticated.",
    "radius-port": "RADIUS service port number.",
    "h3c-compatibility": "Enable/disable compatibility with the H3C, a mechanism that performs security checking for authentication.",
    "auth-type": "Authentication methods/protocols permitted for this RADIUS server.",
    "source-ip": "Source IP address for communications to the RADIUS server.",
    "source-ip-interface": "Source interface for communication with the RADIUS server.",
    "username-case-sensitive": "Enable/disable case sensitive user names.",
    "group-override-attr-type": "RADIUS attribute type to override user group information.",
    "class": "Class attribute name(s).",
    "password-renewal": "Enable/disable password renewal.",
    "require-message-authenticator": "Require message authenticator in authentication response.",
    "password-encoding": "Password encoding.",
    "mac-username-delimiter": "MAC authentication username delimiter (default = hyphen).",
    "mac-password-delimiter": "MAC authentication password delimiter (default = hyphen).",
    "mac-case": "MAC authentication case (default = lowercase).",
    "acct-all-servers": "Enable/disable sending of accounting messages to all configured servers (default = disable).",
    "switch-controller-acct-fast-framedip-detect": "Switch controller accounting message Framed-IP detection from DHCP snooping (seconds, default=2).",
    "interface-select-method": "Specify how to select outgoing interface to reach server.",
    "interface": "Specify outgoing interface to reach server.",
    "vrf-select": "VRF ID used for connection to server.",
    "switch-controller-service-type": "RADIUS service type.",
    "transport-protocol": "Transport protocol to be used (default = udp).",
    "tls-min-proto-version": "Minimum supported protocol version for TLS connections (default is to follow system global setting).",
    "ca-cert": "CA of server to trust under TLS.",
    "client-cert": "Client certificate to use under TLS.",
    "server-identity-check": "Enable/disable RADIUS server identity check (verify server domain name/IP address against the server certificate).",
    "account-key-processing": "Account key processing operation. The FortiGate will keep either the whole domain or strip the domain from the subject identity.",
    "account-key-cert-field": "Define subject identity field in certificate for user access right checking.",
    "rsso": "Enable/disable RADIUS based single sign on feature.",
    "rsso-radius-server-port": "UDP port to listen on for RADIUS Start and Stop records.",
    "rsso-radius-response": "Enable/disable sending RADIUS response packets after receiving Start and Stop records.",
    "rsso-validate-request-secret": "Enable/disable validating the RADIUS request shared secret in the Start or End record.",
    "rsso-secret": "RADIUS secret used by the RADIUS accounting server.",
    "rsso-endpoint-attribute": "RADIUS attributes used to extract the user end point identifier from the RADIUS Start record.",
    "rsso-endpoint-block-attribute": "RADIUS attributes used to block a user.",
    "sso-attribute": "RADIUS attribute that contains the profile group name to be extracted from the RADIUS Start record.",
    "sso-attribute-key": "Key prefix for SSO group value in the SSO attribute.",
    "sso-attribute-value-override": "Enable/disable override old attribute value with new value for the same endpoint.",
    "rsso-context-timeout": "Time in seconds before the logged out user is removed from the \"user context list\" of logged on users.",
    "rsso-log-period": "Time interval in seconds that group event log messages will be generated for dynamic profile events.",
    "rsso-log-flags": "Events to log.",
    "rsso-flush-ip-session": "Enable/disable flushing user IP sessions on RADIUS accounting Stop messages.",
    "rsso-ep-one-ip-only": "Enable/disable the replacement of old IP addresses with new ones for the same endpoint on RADIUS accounting Start messages.",
    "delimiter": "Configure delimiter to be used for separating profile group names in the SSO attribute (default = plus character \"+\").",
    "accounting-server": "Additional accounting servers.",
}

# Field constraints (string lengths, integer ranges)
FIELD_CONSTRAINTS = {
    "name": {"type": "string", "max_length": 35},
    "server": {"type": "string", "max_length": 63},
    "secondary-server": {"type": "string", "max_length": 63},
    "tertiary-server": {"type": "string", "max_length": 63},
    "timeout": {"type": "integer", "min": 1, "max": 300},
    "status-ttl": {"type": "integer", "min": 0, "max": 600},
    "nas-id": {"type": "string", "max_length": 255},
    "acct-interim-interval": {"type": "integer", "min": 60, "max": 86400},
    "radius-port": {"type": "integer", "min": 0, "max": 65535},
    "source-ip": {"type": "string", "max_length": 63},
    "source-ip-interface": {"type": "string", "max_length": 15},
    "switch-controller-acct-fast-framedip-detect": {"type": "integer", "min": 2, "max": 600},
    "interface": {"type": "string", "max_length": 15},
    "vrf-select": {"type": "integer", "min": 0, "max": 511},
    "ca-cert": {"type": "string", "max_length": 79},
    "client-cert": {"type": "string", "max_length": 35},
    "rsso-radius-server-port": {"type": "integer", "min": 0, "max": 65535},
    "sso-attribute-key": {"type": "string", "max_length": 35},
    "rsso-context-timeout": {"type": "integer", "min": 0, "max": 4294967295},
    "rsso-log-period": {"type": "integer", "min": 0, "max": 4294967295},
}

# Nested schemas (for table/list fields with children)
NESTED_SCHEMAS = {
    "class": {
        "name": {
            "type": "string",
            "help": "Class name.",
            "default": "",
            "max_length": 79,
        },
    },
    "accounting-server": {
        "id": {
            "type": "integer",
            "help": "ID (0 - 4294967295).",
            "default": 0,
            "min_value": 0,
            "max_value": 4294967295,
        },
        "status": {
            "type": "option",
            "help": "Status.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "server": {
            "type": "string",
            "help": "Server CN domain name or IP address.",
            "required": True,
            "default": "",
            "max_length": 63,
        },
        "secret": {
            "type": "password",
            "help": "Secret key.",
            "required": True,
            "max_length": 128,
        },
        "port": {
            "type": "integer",
            "help": "RADIUS accounting port number.",
            "default": 0,
            "min_value": 0,
            "max_value": 65535,
        },
        "source-ip": {
            "type": "string",
            "help": "Source IP address for communications to the RADIUS server.",
            "default": "",
            "max_length": 63,
        },
        "interface-select-method": {
            "type": "option",
            "help": "Specify how to select outgoing interface to reach server.",
            "default": "auto",
            "options": ["auto", "sdwan", "specify"],
        },
        "interface": {
            "type": "string",
            "help": "Specify outgoing interface to reach server.",
            "required": True,
            "default": "",
            "max_length": 15,
        },
        "vrf-select": {
            "type": "integer",
            "help": "VRF ID used for connection to server.",
            "default": 0,
            "min_value": 0,
            "max_value": 511,
        },
    },
}


# Valid enum values from API documentation
VALID_BODY_ALL_USERGROUP = [
    "disable",
    "enable",
]
VALID_BODY_USE_MANAGEMENT_VDOM = [
    "enable",
    "disable",
]
VALID_BODY_SWITCH_CONTROLLER_NAS_IP_DYNAMIC = [
    "enable",
    "disable",
]
VALID_BODY_NAS_ID_TYPE = [
    "legacy",
    "custom",
    "hostname",
]
VALID_BODY_CALL_STATION_ID_TYPE = [
    "legacy",
    "IP",
    "MAC",
]
VALID_BODY_RADIUS_COA = [
    "enable",
    "disable",
]
VALID_BODY_H3C_COMPATIBILITY = [
    "enable",
    "disable",
]
VALID_BODY_AUTH_TYPE = [
    "auto",
    "ms_chap_v2",
    "ms_chap",
    "chap",
    "pap",
]
VALID_BODY_USERNAME_CASE_SENSITIVE = [
    "enable",
    "disable",
]
VALID_BODY_GROUP_OVERRIDE_ATTR_TYPE = [
    "filter-Id",
    "class",
]
VALID_BODY_PASSWORD_RENEWAL = [
    "enable",
    "disable",
]
VALID_BODY_REQUIRE_MESSAGE_AUTHENTICATOR = [
    "enable",
    "disable",
]
VALID_BODY_PASSWORD_ENCODING = [
    "auto",
    "ISO-8859-1",
]
VALID_BODY_MAC_USERNAME_DELIMITER = [
    "hyphen",
    "single-hyphen",
    "colon",
    "none",
]
VALID_BODY_MAC_PASSWORD_DELIMITER = [
    "hyphen",
    "single-hyphen",
    "colon",
    "none",
]
VALID_BODY_MAC_CASE = [
    "uppercase",
    "lowercase",
]
VALID_BODY_ACCT_ALL_SERVERS = [
    "enable",
    "disable",
]
VALID_BODY_INTERFACE_SELECT_METHOD = [
    "auto",
    "sdwan",
    "specify",
]
VALID_BODY_SWITCH_CONTROLLER_SERVICE_TYPE = [
    "login",
    "framed",
    "callback-login",
    "callback-framed",
    "outbound",
    "administrative",
    "nas-prompt",
    "authenticate-only",
    "callback-nas-prompt",
    "call-check",
    "callback-administrative",
]
VALID_BODY_TRANSPORT_PROTOCOL = [
    "udp",
    "tcp",
    "tls",
]
VALID_BODY_TLS_MIN_PROTO_VERSION = [
    "default",
    "SSLv3",
    "TLSv1",
    "TLSv1-1",
    "TLSv1-2",
    "TLSv1-3",
]
VALID_BODY_SERVER_IDENTITY_CHECK = [
    "enable",
    "disable",
]
VALID_BODY_ACCOUNT_KEY_PROCESSING = [
    "same",
    "strip",
]
VALID_BODY_ACCOUNT_KEY_CERT_FIELD = [
    "othername",
    "rfc822name",
    "dnsname",
    "cn",
]
VALID_BODY_RSSO = [
    "enable",
    "disable",
]
VALID_BODY_RSSO_RADIUS_RESPONSE = [
    "enable",
    "disable",
]
VALID_BODY_RSSO_VALIDATE_REQUEST_SECRET = [
    "enable",
    "disable",
]
VALID_BODY_RSSO_ENDPOINT_ATTRIBUTE = [
    "User-Name",
    "NAS-IP-Address",
    "Framed-IP-Address",
    "Framed-IP-Netmask",
    "Filter-Id",
    "Login-IP-Host",
    "Reply-Message",
    "Callback-Number",
    "Callback-Id",
    "Framed-Route",
    "Framed-IPX-Network",
    "Class",
    "Called-Station-Id",
    "Calling-Station-Id",
    "NAS-Identifier",
    "Proxy-State",
    "Login-LAT-Service",
    "Login-LAT-Node",
    "Login-LAT-Group",
    "Framed-AppleTalk-Zone",
    "Acct-Session-Id",
    "Acct-Multi-Session-Id",
]
VALID_BODY_RSSO_ENDPOINT_BLOCK_ATTRIBUTE = [
    "User-Name",
    "NAS-IP-Address",
    "Framed-IP-Address",
    "Framed-IP-Netmask",
    "Filter-Id",
    "Login-IP-Host",
    "Reply-Message",
    "Callback-Number",
    "Callback-Id",
    "Framed-Route",
    "Framed-IPX-Network",
    "Class",
    "Called-Station-Id",
    "Calling-Station-Id",
    "NAS-Identifier",
    "Proxy-State",
    "Login-LAT-Service",
    "Login-LAT-Node",
    "Login-LAT-Group",
    "Framed-AppleTalk-Zone",
    "Acct-Session-Id",
    "Acct-Multi-Session-Id",
]
VALID_BODY_SSO_ATTRIBUTE = [
    "User-Name",
    "NAS-IP-Address",
    "Framed-IP-Address",
    "Framed-IP-Netmask",
    "Filter-Id",
    "Login-IP-Host",
    "Reply-Message",
    "Callback-Number",
    "Callback-Id",
    "Framed-Route",
    "Framed-IPX-Network",
    "Class",
    "Called-Station-Id",
    "Calling-Station-Id",
    "NAS-Identifier",
    "Proxy-State",
    "Login-LAT-Service",
    "Login-LAT-Node",
    "Login-LAT-Group",
    "Framed-AppleTalk-Zone",
    "Acct-Session-Id",
    "Acct-Multi-Session-Id",
]
VALID_BODY_SSO_ATTRIBUTE_VALUE_OVERRIDE = [
    "enable",
    "disable",
]
VALID_BODY_RSSO_LOG_FLAGS = [
    "protocol-error",
    "profile-missing",
    "accounting-stop-missed",
    "accounting-event",
    "endpoint-block",
    "radiusd-other",
    "none",
]
VALID_BODY_RSSO_FLUSH_IP_SESSION = [
    "enable",
    "disable",
]
VALID_BODY_RSSO_EP_ONE_IP_ONLY = [
    "enable",
    "disable",
]
VALID_BODY_DELIMITER = [
    "plus",
    "comma",
]
VALID_QUERY_ACTION = ["default", "schema"]

# ============================================================================
# GET Validation
# ============================================================================


def validate_user_radius_get(
    attr: str | None = None,
    filters: dict[str, Any] | None = None,
    **params: Any,
) -> tuple[bool, str | None]:
    """
    Validate GET request parameters for user/radius.

    Args:
        attr: Attribute filter (optional)
        filters: Additional filter parameters
        **params: Other query parameters

    Returns:
        Tuple of (is_valid, error_message)

    Examples:
        >>> # Valid - Get all items
        >>> is_valid, error = validate_user_radius_get()
        >>> assert is_valid == True
        
        >>> # Valid - Get specific item by name
        >>> is_valid, error = validate_user_radius_get(name="test-item")
        >>> assert is_valid == True
        
        >>> # Valid - With filters
        >>> is_valid, error = validate_user_radius_get(
        ...     filters={"format": "name|type"}
        ... )
        >>> assert is_valid == True
    """
    # Validate query parameters if present
    if "action" in params:
        value = params.get("action")
        if value and value not in VALID_QUERY_ACTION:
            return (
                False,
                f"Invalid query parameter 'action'='{value}'. Must be one of: {', '.join(VALID_QUERY_ACTION)}",
            )

    return (True, None)


# ============================================================================
# POST Validation
# ============================================================================


def validate_required_fields(payload: dict) -> tuple[bool, str | None]:
    """
    Validate required fields for user/radius.

    This validator checks:
    1. Always-required fields are present
    2. Mutually exclusive groups have at least one field

    Args:
        payload: The request payload to validate

    Returns:
        Tuple of (is_valid, error_message)

    Example:
        >>> payload = {"name": "test"}
        >>> is_valid, error = validate_required_fields(payload)
    """
    # Check always-required fields
    missing_fields = []
    for field in REQUIRED_FIELDS:
        if field not in payload:
            missing_fields.append(field)
    
    if missing_fields:
        # Build enhanced error message
        error_parts = [f"Missing required field(s): {', '.join(missing_fields)}"]
        
        # Add descriptions for first few missing fields
        for field in missing_fields[:3]:
            desc = FIELD_DESCRIPTIONS.get(field)
            if desc:
                error_parts.append(f"  • {field}: {desc}")
        
        if len(missing_fields) > 3:
            error_parts.append(f"  ... and {len(missing_fields) - 3} more")
        
        return (False, "\n".join(error_parts))

    return (True, None)


def validate_user_radius_post(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """
    Validate POST request to create new user/radius object.

    This validator performs two-stage validation:
    1. Required fields check (schema-based)
    2. Field value validation (enums, ranges, formats)

    Args:
        payload: Request body data with configuration
        **params: Query parameters (vdom, etc.)

    Returns:
        Tuple of (is_valid, error_message)
        - is_valid: True if payload is valid, False otherwise
        - error_message: None if valid, detailed error string if invalid

    Examples:
        >>> # ✅ Valid - Minimal required fields
        >>> payload = {
        ...     "server": True,  # Primary RADIUS server CN domain name or IP address
        ...     "secret": True,  # Pre-shared secret key used to access the primary R
        ... }
        >>> is_valid, error = validate_user_radius_post(payload)
        >>> assert is_valid == True
        
        >>> # ✅ Valid - With enum field
        >>> payload = {
        ...     "server": True,
        ...     "all-usergroup": "disable",  # Valid enum value
        ... }
        >>> is_valid, error = validate_user_radius_post(payload)
        >>> assert is_valid == True
        
        >>> # ❌ Invalid - Wrong enum value
        >>> payload["all-usergroup"] = "invalid-value"
        >>> is_valid, error = validate_user_radius_post(payload)
        >>> assert is_valid == False
        >>> assert "Invalid value" in error
        
        >>> # ❌ Invalid - Missing required field
        >>> payload = {}  # Empty payload
        >>> is_valid, error = validate_user_radius_post(payload)
        >>> assert is_valid == False
        >>> assert "Missing required field" in error
    """
    # Step 1: Validate required fields
    is_valid, error = validate_required_fields(payload)
    if not is_valid:
        return (False, error)

    # Step 2: Validate enum values
    if "all-usergroup" in payload:
        value = payload["all-usergroup"]
        if value not in VALID_BODY_ALL_USERGROUP:
            desc = FIELD_DESCRIPTIONS.get("all-usergroup", "")
            error_msg = f"Invalid value for 'all-usergroup': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_ALL_USERGROUP)}"
            error_msg += f"\n  → Example: all-usergroup='{{ VALID_BODY_ALL_USERGROUP[0] }}'"
            return (False, error_msg)
    if "use-management-vdom" in payload:
        value = payload["use-management-vdom"]
        if value not in VALID_BODY_USE_MANAGEMENT_VDOM:
            desc = FIELD_DESCRIPTIONS.get("use-management-vdom", "")
            error_msg = f"Invalid value for 'use-management-vdom': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_USE_MANAGEMENT_VDOM)}"
            error_msg += f"\n  → Example: use-management-vdom='{{ VALID_BODY_USE_MANAGEMENT_VDOM[0] }}'"
            return (False, error_msg)
    if "switch-controller-nas-ip-dynamic" in payload:
        value = payload["switch-controller-nas-ip-dynamic"]
        if value not in VALID_BODY_SWITCH_CONTROLLER_NAS_IP_DYNAMIC:
            desc = FIELD_DESCRIPTIONS.get("switch-controller-nas-ip-dynamic", "")
            error_msg = f"Invalid value for 'switch-controller-nas-ip-dynamic': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_SWITCH_CONTROLLER_NAS_IP_DYNAMIC)}"
            error_msg += f"\n  → Example: switch-controller-nas-ip-dynamic='{{ VALID_BODY_SWITCH_CONTROLLER_NAS_IP_DYNAMIC[0] }}'"
            return (False, error_msg)
    if "nas-id-type" in payload:
        value = payload["nas-id-type"]
        if value not in VALID_BODY_NAS_ID_TYPE:
            desc = FIELD_DESCRIPTIONS.get("nas-id-type", "")
            error_msg = f"Invalid value for 'nas-id-type': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_NAS_ID_TYPE)}"
            error_msg += f"\n  → Example: nas-id-type='{{ VALID_BODY_NAS_ID_TYPE[0] }}'"
            return (False, error_msg)
    if "call-station-id-type" in payload:
        value = payload["call-station-id-type"]
        if value not in VALID_BODY_CALL_STATION_ID_TYPE:
            desc = FIELD_DESCRIPTIONS.get("call-station-id-type", "")
            error_msg = f"Invalid value for 'call-station-id-type': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_CALL_STATION_ID_TYPE)}"
            error_msg += f"\n  → Example: call-station-id-type='{{ VALID_BODY_CALL_STATION_ID_TYPE[0] }}'"
            return (False, error_msg)
    if "radius-coa" in payload:
        value = payload["radius-coa"]
        if value not in VALID_BODY_RADIUS_COA:
            desc = FIELD_DESCRIPTIONS.get("radius-coa", "")
            error_msg = f"Invalid value for 'radius-coa': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_RADIUS_COA)}"
            error_msg += f"\n  → Example: radius-coa='{{ VALID_BODY_RADIUS_COA[0] }}'"
            return (False, error_msg)
    if "h3c-compatibility" in payload:
        value = payload["h3c-compatibility"]
        if value not in VALID_BODY_H3C_COMPATIBILITY:
            desc = FIELD_DESCRIPTIONS.get("h3c-compatibility", "")
            error_msg = f"Invalid value for 'h3c-compatibility': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_H3C_COMPATIBILITY)}"
            error_msg += f"\n  → Example: h3c-compatibility='{{ VALID_BODY_H3C_COMPATIBILITY[0] }}'"
            return (False, error_msg)
    if "auth-type" in payload:
        value = payload["auth-type"]
        if value not in VALID_BODY_AUTH_TYPE:
            desc = FIELD_DESCRIPTIONS.get("auth-type", "")
            error_msg = f"Invalid value for 'auth-type': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_AUTH_TYPE)}"
            error_msg += f"\n  → Example: auth-type='{{ VALID_BODY_AUTH_TYPE[0] }}'"
            return (False, error_msg)
    if "username-case-sensitive" in payload:
        value = payload["username-case-sensitive"]
        if value not in VALID_BODY_USERNAME_CASE_SENSITIVE:
            desc = FIELD_DESCRIPTIONS.get("username-case-sensitive", "")
            error_msg = f"Invalid value for 'username-case-sensitive': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_USERNAME_CASE_SENSITIVE)}"
            error_msg += f"\n  → Example: username-case-sensitive='{{ VALID_BODY_USERNAME_CASE_SENSITIVE[0] }}'"
            return (False, error_msg)
    if "group-override-attr-type" in payload:
        value = payload["group-override-attr-type"]
        if value not in VALID_BODY_GROUP_OVERRIDE_ATTR_TYPE:
            desc = FIELD_DESCRIPTIONS.get("group-override-attr-type", "")
            error_msg = f"Invalid value for 'group-override-attr-type': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_GROUP_OVERRIDE_ATTR_TYPE)}"
            error_msg += f"\n  → Example: group-override-attr-type='{{ VALID_BODY_GROUP_OVERRIDE_ATTR_TYPE[0] }}'"
            return (False, error_msg)
    if "password-renewal" in payload:
        value = payload["password-renewal"]
        if value not in VALID_BODY_PASSWORD_RENEWAL:
            desc = FIELD_DESCRIPTIONS.get("password-renewal", "")
            error_msg = f"Invalid value for 'password-renewal': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_PASSWORD_RENEWAL)}"
            error_msg += f"\n  → Example: password-renewal='{{ VALID_BODY_PASSWORD_RENEWAL[0] }}'"
            return (False, error_msg)
    if "require-message-authenticator" in payload:
        value = payload["require-message-authenticator"]
        if value not in VALID_BODY_REQUIRE_MESSAGE_AUTHENTICATOR:
            desc = FIELD_DESCRIPTIONS.get("require-message-authenticator", "")
            error_msg = f"Invalid value for 'require-message-authenticator': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_REQUIRE_MESSAGE_AUTHENTICATOR)}"
            error_msg += f"\n  → Example: require-message-authenticator='{{ VALID_BODY_REQUIRE_MESSAGE_AUTHENTICATOR[0] }}'"
            return (False, error_msg)
    if "password-encoding" in payload:
        value = payload["password-encoding"]
        if value not in VALID_BODY_PASSWORD_ENCODING:
            desc = FIELD_DESCRIPTIONS.get("password-encoding", "")
            error_msg = f"Invalid value for 'password-encoding': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_PASSWORD_ENCODING)}"
            error_msg += f"\n  → Example: password-encoding='{{ VALID_BODY_PASSWORD_ENCODING[0] }}'"
            return (False, error_msg)
    if "mac-username-delimiter" in payload:
        value = payload["mac-username-delimiter"]
        if value not in VALID_BODY_MAC_USERNAME_DELIMITER:
            desc = FIELD_DESCRIPTIONS.get("mac-username-delimiter", "")
            error_msg = f"Invalid value for 'mac-username-delimiter': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_MAC_USERNAME_DELIMITER)}"
            error_msg += f"\n  → Example: mac-username-delimiter='{{ VALID_BODY_MAC_USERNAME_DELIMITER[0] }}'"
            return (False, error_msg)
    if "mac-password-delimiter" in payload:
        value = payload["mac-password-delimiter"]
        if value not in VALID_BODY_MAC_PASSWORD_DELIMITER:
            desc = FIELD_DESCRIPTIONS.get("mac-password-delimiter", "")
            error_msg = f"Invalid value for 'mac-password-delimiter': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_MAC_PASSWORD_DELIMITER)}"
            error_msg += f"\n  → Example: mac-password-delimiter='{{ VALID_BODY_MAC_PASSWORD_DELIMITER[0] }}'"
            return (False, error_msg)
    if "mac-case" in payload:
        value = payload["mac-case"]
        if value not in VALID_BODY_MAC_CASE:
            desc = FIELD_DESCRIPTIONS.get("mac-case", "")
            error_msg = f"Invalid value for 'mac-case': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_MAC_CASE)}"
            error_msg += f"\n  → Example: mac-case='{{ VALID_BODY_MAC_CASE[0] }}'"
            return (False, error_msg)
    if "acct-all-servers" in payload:
        value = payload["acct-all-servers"]
        if value not in VALID_BODY_ACCT_ALL_SERVERS:
            desc = FIELD_DESCRIPTIONS.get("acct-all-servers", "")
            error_msg = f"Invalid value for 'acct-all-servers': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_ACCT_ALL_SERVERS)}"
            error_msg += f"\n  → Example: acct-all-servers='{{ VALID_BODY_ACCT_ALL_SERVERS[0] }}'"
            return (False, error_msg)
    if "interface-select-method" in payload:
        value = payload["interface-select-method"]
        if value not in VALID_BODY_INTERFACE_SELECT_METHOD:
            desc = FIELD_DESCRIPTIONS.get("interface-select-method", "")
            error_msg = f"Invalid value for 'interface-select-method': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_INTERFACE_SELECT_METHOD)}"
            error_msg += f"\n  → Example: interface-select-method='{{ VALID_BODY_INTERFACE_SELECT_METHOD[0] }}'"
            return (False, error_msg)
    if "switch-controller-service-type" in payload:
        value = payload["switch-controller-service-type"]
        if value not in VALID_BODY_SWITCH_CONTROLLER_SERVICE_TYPE:
            desc = FIELD_DESCRIPTIONS.get("switch-controller-service-type", "")
            error_msg = f"Invalid value for 'switch-controller-service-type': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_SWITCH_CONTROLLER_SERVICE_TYPE)}"
            error_msg += f"\n  → Example: switch-controller-service-type='{{ VALID_BODY_SWITCH_CONTROLLER_SERVICE_TYPE[0] }}'"
            return (False, error_msg)
    if "transport-protocol" in payload:
        value = payload["transport-protocol"]
        if value not in VALID_BODY_TRANSPORT_PROTOCOL:
            desc = FIELD_DESCRIPTIONS.get("transport-protocol", "")
            error_msg = f"Invalid value for 'transport-protocol': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_TRANSPORT_PROTOCOL)}"
            error_msg += f"\n  → Example: transport-protocol='{{ VALID_BODY_TRANSPORT_PROTOCOL[0] }}'"
            return (False, error_msg)
    if "tls-min-proto-version" in payload:
        value = payload["tls-min-proto-version"]
        if value not in VALID_BODY_TLS_MIN_PROTO_VERSION:
            desc = FIELD_DESCRIPTIONS.get("tls-min-proto-version", "")
            error_msg = f"Invalid value for 'tls-min-proto-version': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_TLS_MIN_PROTO_VERSION)}"
            error_msg += f"\n  → Example: tls-min-proto-version='{{ VALID_BODY_TLS_MIN_PROTO_VERSION[0] }}'"
            return (False, error_msg)
    if "server-identity-check" in payload:
        value = payload["server-identity-check"]
        if value not in VALID_BODY_SERVER_IDENTITY_CHECK:
            desc = FIELD_DESCRIPTIONS.get("server-identity-check", "")
            error_msg = f"Invalid value for 'server-identity-check': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_SERVER_IDENTITY_CHECK)}"
            error_msg += f"\n  → Example: server-identity-check='{{ VALID_BODY_SERVER_IDENTITY_CHECK[0] }}'"
            return (False, error_msg)
    if "account-key-processing" in payload:
        value = payload["account-key-processing"]
        if value not in VALID_BODY_ACCOUNT_KEY_PROCESSING:
            desc = FIELD_DESCRIPTIONS.get("account-key-processing", "")
            error_msg = f"Invalid value for 'account-key-processing': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_ACCOUNT_KEY_PROCESSING)}"
            error_msg += f"\n  → Example: account-key-processing='{{ VALID_BODY_ACCOUNT_KEY_PROCESSING[0] }}'"
            return (False, error_msg)
    if "account-key-cert-field" in payload:
        value = payload["account-key-cert-field"]
        if value not in VALID_BODY_ACCOUNT_KEY_CERT_FIELD:
            desc = FIELD_DESCRIPTIONS.get("account-key-cert-field", "")
            error_msg = f"Invalid value for 'account-key-cert-field': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_ACCOUNT_KEY_CERT_FIELD)}"
            error_msg += f"\n  → Example: account-key-cert-field='{{ VALID_BODY_ACCOUNT_KEY_CERT_FIELD[0] }}'"
            return (False, error_msg)
    if "rsso" in payload:
        value = payload["rsso"]
        if value not in VALID_BODY_RSSO:
            desc = FIELD_DESCRIPTIONS.get("rsso", "")
            error_msg = f"Invalid value for 'rsso': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_RSSO)}"
            error_msg += f"\n  → Example: rsso='{{ VALID_BODY_RSSO[0] }}'"
            return (False, error_msg)
    if "rsso-radius-response" in payload:
        value = payload["rsso-radius-response"]
        if value not in VALID_BODY_RSSO_RADIUS_RESPONSE:
            desc = FIELD_DESCRIPTIONS.get("rsso-radius-response", "")
            error_msg = f"Invalid value for 'rsso-radius-response': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_RSSO_RADIUS_RESPONSE)}"
            error_msg += f"\n  → Example: rsso-radius-response='{{ VALID_BODY_RSSO_RADIUS_RESPONSE[0] }}'"
            return (False, error_msg)
    if "rsso-validate-request-secret" in payload:
        value = payload["rsso-validate-request-secret"]
        if value not in VALID_BODY_RSSO_VALIDATE_REQUEST_SECRET:
            desc = FIELD_DESCRIPTIONS.get("rsso-validate-request-secret", "")
            error_msg = f"Invalid value for 'rsso-validate-request-secret': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_RSSO_VALIDATE_REQUEST_SECRET)}"
            error_msg += f"\n  → Example: rsso-validate-request-secret='{{ VALID_BODY_RSSO_VALIDATE_REQUEST_SECRET[0] }}'"
            return (False, error_msg)
    if "rsso-endpoint-attribute" in payload:
        value = payload["rsso-endpoint-attribute"]
        if value not in VALID_BODY_RSSO_ENDPOINT_ATTRIBUTE:
            desc = FIELD_DESCRIPTIONS.get("rsso-endpoint-attribute", "")
            error_msg = f"Invalid value for 'rsso-endpoint-attribute': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_RSSO_ENDPOINT_ATTRIBUTE)}"
            error_msg += f"\n  → Example: rsso-endpoint-attribute='{{ VALID_BODY_RSSO_ENDPOINT_ATTRIBUTE[0] }}'"
            return (False, error_msg)
    if "rsso-endpoint-block-attribute" in payload:
        value = payload["rsso-endpoint-block-attribute"]
        if value not in VALID_BODY_RSSO_ENDPOINT_BLOCK_ATTRIBUTE:
            desc = FIELD_DESCRIPTIONS.get("rsso-endpoint-block-attribute", "")
            error_msg = f"Invalid value for 'rsso-endpoint-block-attribute': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_RSSO_ENDPOINT_BLOCK_ATTRIBUTE)}"
            error_msg += f"\n  → Example: rsso-endpoint-block-attribute='{{ VALID_BODY_RSSO_ENDPOINT_BLOCK_ATTRIBUTE[0] }}'"
            return (False, error_msg)
    if "sso-attribute" in payload:
        value = payload["sso-attribute"]
        if value not in VALID_BODY_SSO_ATTRIBUTE:
            desc = FIELD_DESCRIPTIONS.get("sso-attribute", "")
            error_msg = f"Invalid value for 'sso-attribute': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_SSO_ATTRIBUTE)}"
            error_msg += f"\n  → Example: sso-attribute='{{ VALID_BODY_SSO_ATTRIBUTE[0] }}'"
            return (False, error_msg)
    if "sso-attribute-value-override" in payload:
        value = payload["sso-attribute-value-override"]
        if value not in VALID_BODY_SSO_ATTRIBUTE_VALUE_OVERRIDE:
            desc = FIELD_DESCRIPTIONS.get("sso-attribute-value-override", "")
            error_msg = f"Invalid value for 'sso-attribute-value-override': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_SSO_ATTRIBUTE_VALUE_OVERRIDE)}"
            error_msg += f"\n  → Example: sso-attribute-value-override='{{ VALID_BODY_SSO_ATTRIBUTE_VALUE_OVERRIDE[0] }}'"
            return (False, error_msg)
    if "rsso-log-flags" in payload:
        value = payload["rsso-log-flags"]
        if value not in VALID_BODY_RSSO_LOG_FLAGS:
            desc = FIELD_DESCRIPTIONS.get("rsso-log-flags", "")
            error_msg = f"Invalid value for 'rsso-log-flags': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_RSSO_LOG_FLAGS)}"
            error_msg += f"\n  → Example: rsso-log-flags='{{ VALID_BODY_RSSO_LOG_FLAGS[0] }}'"
            return (False, error_msg)
    if "rsso-flush-ip-session" in payload:
        value = payload["rsso-flush-ip-session"]
        if value not in VALID_BODY_RSSO_FLUSH_IP_SESSION:
            desc = FIELD_DESCRIPTIONS.get("rsso-flush-ip-session", "")
            error_msg = f"Invalid value for 'rsso-flush-ip-session': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_RSSO_FLUSH_IP_SESSION)}"
            error_msg += f"\n  → Example: rsso-flush-ip-session='{{ VALID_BODY_RSSO_FLUSH_IP_SESSION[0] }}'"
            return (False, error_msg)
    if "rsso-ep-one-ip-only" in payload:
        value = payload["rsso-ep-one-ip-only"]
        if value not in VALID_BODY_RSSO_EP_ONE_IP_ONLY:
            desc = FIELD_DESCRIPTIONS.get("rsso-ep-one-ip-only", "")
            error_msg = f"Invalid value for 'rsso-ep-one-ip-only': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_RSSO_EP_ONE_IP_ONLY)}"
            error_msg += f"\n  → Example: rsso-ep-one-ip-only='{{ VALID_BODY_RSSO_EP_ONE_IP_ONLY[0] }}'"
            return (False, error_msg)
    if "delimiter" in payload:
        value = payload["delimiter"]
        if value not in VALID_BODY_DELIMITER:
            desc = FIELD_DESCRIPTIONS.get("delimiter", "")
            error_msg = f"Invalid value for 'delimiter': '{value}'"
            if desc:
                error_msg += f"\n  → Description: {desc}"
            error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in VALID_BODY_DELIMITER)}"
            error_msg += f"\n  → Example: delimiter='{{ VALID_BODY_DELIMITER[0] }}'"
            return (False, error_msg)

    return (True, None)


# ============================================================================
# PUT Validation
# ============================================================================


def validate_user_radius_put(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """
    Validate PUT request to update user/radius.

    Args:
        payload: Request body data
        **params: Query parameters

    Returns:
        Tuple of (is_valid, error_message)

    Example:
        >>> payload = {"name": "updated_item"}
        >>> is_valid, error = validate_user_radius_put(payload)
    """
    # Step 1: Validate enum values
    if "all-usergroup" in payload:
        value = payload["all-usergroup"]
        if value not in VALID_BODY_ALL_USERGROUP:
            return (
                False,
                f"Invalid value for 'all-usergroup'='{value}'. Must be one of: {', '.join(VALID_BODY_ALL_USERGROUP)}",
            )
    if "use-management-vdom" in payload:
        value = payload["use-management-vdom"]
        if value not in VALID_BODY_USE_MANAGEMENT_VDOM:
            return (
                False,
                f"Invalid value for 'use-management-vdom'='{value}'. Must be one of: {', '.join(VALID_BODY_USE_MANAGEMENT_VDOM)}",
            )
    if "switch-controller-nas-ip-dynamic" in payload:
        value = payload["switch-controller-nas-ip-dynamic"]
        if value not in VALID_BODY_SWITCH_CONTROLLER_NAS_IP_DYNAMIC:
            return (
                False,
                f"Invalid value for 'switch-controller-nas-ip-dynamic'='{value}'. Must be one of: {', '.join(VALID_BODY_SWITCH_CONTROLLER_NAS_IP_DYNAMIC)}",
            )
    if "nas-id-type" in payload:
        value = payload["nas-id-type"]
        if value not in VALID_BODY_NAS_ID_TYPE:
            return (
                False,
                f"Invalid value for 'nas-id-type'='{value}'. Must be one of: {', '.join(VALID_BODY_NAS_ID_TYPE)}",
            )
    if "call-station-id-type" in payload:
        value = payload["call-station-id-type"]
        if value not in VALID_BODY_CALL_STATION_ID_TYPE:
            return (
                False,
                f"Invalid value for 'call-station-id-type'='{value}'. Must be one of: {', '.join(VALID_BODY_CALL_STATION_ID_TYPE)}",
            )
    if "radius-coa" in payload:
        value = payload["radius-coa"]
        if value not in VALID_BODY_RADIUS_COA:
            return (
                False,
                f"Invalid value for 'radius-coa'='{value}'. Must be one of: {', '.join(VALID_BODY_RADIUS_COA)}",
            )
    if "h3c-compatibility" in payload:
        value = payload["h3c-compatibility"]
        if value not in VALID_BODY_H3C_COMPATIBILITY:
            return (
                False,
                f"Invalid value for 'h3c-compatibility'='{value}'. Must be one of: {', '.join(VALID_BODY_H3C_COMPATIBILITY)}",
            )
    if "auth-type" in payload:
        value = payload["auth-type"]
        if value not in VALID_BODY_AUTH_TYPE:
            return (
                False,
                f"Invalid value for 'auth-type'='{value}'. Must be one of: {', '.join(VALID_BODY_AUTH_TYPE)}",
            )
    if "username-case-sensitive" in payload:
        value = payload["username-case-sensitive"]
        if value not in VALID_BODY_USERNAME_CASE_SENSITIVE:
            return (
                False,
                f"Invalid value for 'username-case-sensitive'='{value}'. Must be one of: {', '.join(VALID_BODY_USERNAME_CASE_SENSITIVE)}",
            )
    if "group-override-attr-type" in payload:
        value = payload["group-override-attr-type"]
        if value not in VALID_BODY_GROUP_OVERRIDE_ATTR_TYPE:
            return (
                False,
                f"Invalid value for 'group-override-attr-type'='{value}'. Must be one of: {', '.join(VALID_BODY_GROUP_OVERRIDE_ATTR_TYPE)}",
            )
    if "password-renewal" in payload:
        value = payload["password-renewal"]
        if value not in VALID_BODY_PASSWORD_RENEWAL:
            return (
                False,
                f"Invalid value for 'password-renewal'='{value}'. Must be one of: {', '.join(VALID_BODY_PASSWORD_RENEWAL)}",
            )
    if "require-message-authenticator" in payload:
        value = payload["require-message-authenticator"]
        if value not in VALID_BODY_REQUIRE_MESSAGE_AUTHENTICATOR:
            return (
                False,
                f"Invalid value for 'require-message-authenticator'='{value}'. Must be one of: {', '.join(VALID_BODY_REQUIRE_MESSAGE_AUTHENTICATOR)}",
            )
    if "password-encoding" in payload:
        value = payload["password-encoding"]
        if value not in VALID_BODY_PASSWORD_ENCODING:
            return (
                False,
                f"Invalid value for 'password-encoding'='{value}'. Must be one of: {', '.join(VALID_BODY_PASSWORD_ENCODING)}",
            )
    if "mac-username-delimiter" in payload:
        value = payload["mac-username-delimiter"]
        if value not in VALID_BODY_MAC_USERNAME_DELIMITER:
            return (
                False,
                f"Invalid value for 'mac-username-delimiter'='{value}'. Must be one of: {', '.join(VALID_BODY_MAC_USERNAME_DELIMITER)}",
            )
    if "mac-password-delimiter" in payload:
        value = payload["mac-password-delimiter"]
        if value not in VALID_BODY_MAC_PASSWORD_DELIMITER:
            return (
                False,
                f"Invalid value for 'mac-password-delimiter'='{value}'. Must be one of: {', '.join(VALID_BODY_MAC_PASSWORD_DELIMITER)}",
            )
    if "mac-case" in payload:
        value = payload["mac-case"]
        if value not in VALID_BODY_MAC_CASE:
            return (
                False,
                f"Invalid value for 'mac-case'='{value}'. Must be one of: {', '.join(VALID_BODY_MAC_CASE)}",
            )
    if "acct-all-servers" in payload:
        value = payload["acct-all-servers"]
        if value not in VALID_BODY_ACCT_ALL_SERVERS:
            return (
                False,
                f"Invalid value for 'acct-all-servers'='{value}'. Must be one of: {', '.join(VALID_BODY_ACCT_ALL_SERVERS)}",
            )
    if "interface-select-method" in payload:
        value = payload["interface-select-method"]
        if value not in VALID_BODY_INTERFACE_SELECT_METHOD:
            return (
                False,
                f"Invalid value for 'interface-select-method'='{value}'. Must be one of: {', '.join(VALID_BODY_INTERFACE_SELECT_METHOD)}",
            )
    if "switch-controller-service-type" in payload:
        value = payload["switch-controller-service-type"]
        if value not in VALID_BODY_SWITCH_CONTROLLER_SERVICE_TYPE:
            return (
                False,
                f"Invalid value for 'switch-controller-service-type'='{value}'. Must be one of: {', '.join(VALID_BODY_SWITCH_CONTROLLER_SERVICE_TYPE)}",
            )
    if "transport-protocol" in payload:
        value = payload["transport-protocol"]
        if value not in VALID_BODY_TRANSPORT_PROTOCOL:
            return (
                False,
                f"Invalid value for 'transport-protocol'='{value}'. Must be one of: {', '.join(VALID_BODY_TRANSPORT_PROTOCOL)}",
            )
    if "tls-min-proto-version" in payload:
        value = payload["tls-min-proto-version"]
        if value not in VALID_BODY_TLS_MIN_PROTO_VERSION:
            return (
                False,
                f"Invalid value for 'tls-min-proto-version'='{value}'. Must be one of: {', '.join(VALID_BODY_TLS_MIN_PROTO_VERSION)}",
            )
    if "server-identity-check" in payload:
        value = payload["server-identity-check"]
        if value not in VALID_BODY_SERVER_IDENTITY_CHECK:
            return (
                False,
                f"Invalid value for 'server-identity-check'='{value}'. Must be one of: {', '.join(VALID_BODY_SERVER_IDENTITY_CHECK)}",
            )
    if "account-key-processing" in payload:
        value = payload["account-key-processing"]
        if value not in VALID_BODY_ACCOUNT_KEY_PROCESSING:
            return (
                False,
                f"Invalid value for 'account-key-processing'='{value}'. Must be one of: {', '.join(VALID_BODY_ACCOUNT_KEY_PROCESSING)}",
            )
    if "account-key-cert-field" in payload:
        value = payload["account-key-cert-field"]
        if value not in VALID_BODY_ACCOUNT_KEY_CERT_FIELD:
            return (
                False,
                f"Invalid value for 'account-key-cert-field'='{value}'. Must be one of: {', '.join(VALID_BODY_ACCOUNT_KEY_CERT_FIELD)}",
            )
    if "rsso" in payload:
        value = payload["rsso"]
        if value not in VALID_BODY_RSSO:
            return (
                False,
                f"Invalid value for 'rsso'='{value}'. Must be one of: {', '.join(VALID_BODY_RSSO)}",
            )
    if "rsso-radius-response" in payload:
        value = payload["rsso-radius-response"]
        if value not in VALID_BODY_RSSO_RADIUS_RESPONSE:
            return (
                False,
                f"Invalid value for 'rsso-radius-response'='{value}'. Must be one of: {', '.join(VALID_BODY_RSSO_RADIUS_RESPONSE)}",
            )
    if "rsso-validate-request-secret" in payload:
        value = payload["rsso-validate-request-secret"]
        if value not in VALID_BODY_RSSO_VALIDATE_REQUEST_SECRET:
            return (
                False,
                f"Invalid value for 'rsso-validate-request-secret'='{value}'. Must be one of: {', '.join(VALID_BODY_RSSO_VALIDATE_REQUEST_SECRET)}",
            )
    if "rsso-endpoint-attribute" in payload:
        value = payload["rsso-endpoint-attribute"]
        if value not in VALID_BODY_RSSO_ENDPOINT_ATTRIBUTE:
            return (
                False,
                f"Invalid value for 'rsso-endpoint-attribute'='{value}'. Must be one of: {', '.join(VALID_BODY_RSSO_ENDPOINT_ATTRIBUTE)}",
            )
    if "rsso-endpoint-block-attribute" in payload:
        value = payload["rsso-endpoint-block-attribute"]
        if value not in VALID_BODY_RSSO_ENDPOINT_BLOCK_ATTRIBUTE:
            return (
                False,
                f"Invalid value for 'rsso-endpoint-block-attribute'='{value}'. Must be one of: {', '.join(VALID_BODY_RSSO_ENDPOINT_BLOCK_ATTRIBUTE)}",
            )
    if "sso-attribute" in payload:
        value = payload["sso-attribute"]
        if value not in VALID_BODY_SSO_ATTRIBUTE:
            return (
                False,
                f"Invalid value for 'sso-attribute'='{value}'. Must be one of: {', '.join(VALID_BODY_SSO_ATTRIBUTE)}",
            )
    if "sso-attribute-value-override" in payload:
        value = payload["sso-attribute-value-override"]
        if value not in VALID_BODY_SSO_ATTRIBUTE_VALUE_OVERRIDE:
            return (
                False,
                f"Invalid value for 'sso-attribute-value-override'='{value}'. Must be one of: {', '.join(VALID_BODY_SSO_ATTRIBUTE_VALUE_OVERRIDE)}",
            )
    if "rsso-log-flags" in payload:
        value = payload["rsso-log-flags"]
        if value not in VALID_BODY_RSSO_LOG_FLAGS:
            return (
                False,
                f"Invalid value for 'rsso-log-flags'='{value}'. Must be one of: {', '.join(VALID_BODY_RSSO_LOG_FLAGS)}",
            )
    if "rsso-flush-ip-session" in payload:
        value = payload["rsso-flush-ip-session"]
        if value not in VALID_BODY_RSSO_FLUSH_IP_SESSION:
            return (
                False,
                f"Invalid value for 'rsso-flush-ip-session'='{value}'. Must be one of: {', '.join(VALID_BODY_RSSO_FLUSH_IP_SESSION)}",
            )
    if "rsso-ep-one-ip-only" in payload:
        value = payload["rsso-ep-one-ip-only"]
        if value not in VALID_BODY_RSSO_EP_ONE_IP_ONLY:
            return (
                False,
                f"Invalid value for 'rsso-ep-one-ip-only'='{value}'. Must be one of: {', '.join(VALID_BODY_RSSO_EP_ONE_IP_ONLY)}",
            )
    if "delimiter" in payload:
        value = payload["delimiter"]
        if value not in VALID_BODY_DELIMITER:
            return (
                False,
                f"Invalid value for 'delimiter'='{value}'. Must be one of: {', '.join(VALID_BODY_DELIMITER)}",
            )

    return (True, None)


# ============================================================================
# Metadata Access Functions
# Provide programmatic access to field metadata for IDE autocomplete,
# documentation generation, and dynamic validation
# ============================================================================


def get_field_description(field_name: str) -> str | None:
    """
    Get description/help text for a field.

    Args:
        field_name: Name of the field

    Returns:
        Description text or None if field doesn't exist

    Example:
        >>> desc = get_field_description("name")
        >>> print(desc)
    """
    return FIELD_DESCRIPTIONS.get(field_name)


def get_field_type(field_name: str) -> str | None:
    """
    Get the type of a field.

    Args:
        field_name: Name of the field

    Returns:
        Field type (e.g., "string", "integer", "option") or None

    Example:
        >>> field_type = get_field_type("status")
        >>> print(field_type)  # "option"
    """
    return FIELD_TYPES.get(field_name)


def get_field_constraints(field_name: str) -> dict[str, Any] | None:
    """
    Get constraints for a field (min/max values, string length).

    Args:
        field_name: Name of the field

    Returns:
        Constraint dict or None

    Example:
        >>> constraints = get_field_constraints("port")
        >>> print(constraints)  # {"type": "integer", "min": 1, "max": 65535}
    """
    return FIELD_CONSTRAINTS.get(field_name)


def get_field_default(field_name: str) -> Any | None:
    """
    Get default value for a field.

    Args:
        field_name: Name of the field

    Returns:
        Default value or None if no default

    Example:
        >>> default = get_field_default("status")
        >>> print(default)  # "enable"
    """
    return FIELDS_WITH_DEFAULTS.get(field_name)


def get_field_options(field_name: str) -> list[str] | None:
    """
    Get valid enum options for a field.

    Args:
        field_name: Name of the field

    Returns:
        List of valid values or None if not an enum field

    Example:
        >>> options = get_field_options("status")
        >>> print(options)  # ["enable", "disable"]
    """
    # Construct the constant name from field name
    # Replace all non-alphanumeric characters with underscores for valid Python identifiers
    import re
    safe_name = re.sub(r'[^a-zA-Z0-9]', '_', field_name)
    constant_name = f"VALID_BODY_{safe_name.upper()}"
    return globals().get(constant_name)


def get_nested_schema(field_name: str) -> dict[str, Any] | None:
    """
    Get schema for nested table/list fields.

    Args:
        field_name: Name of the parent field

    Returns:
        Dict mapping child field names to their metadata

    Example:
        >>> nested = get_nested_schema("members")
        >>> if nested:
        ...     for child_field, child_meta in nested.items():
        ...         print(f"{child_field}: {child_meta['type']}")
    """
    return NESTED_SCHEMAS.get(field_name)


def get_all_fields() -> list[str]:
    """
    Get list of all field names.

    Returns:
        List of all field names in the schema

    Example:
        >>> fields = get_all_fields()
        >>> print(len(fields))
    """
    return list(FIELD_TYPES.keys())


def get_field_metadata(field_name: str) -> dict[str, Any] | None:
    """
    Get complete metadata for a field (type, description, constraints, defaults, options).

    Args:
        field_name: Name of the field

    Returns:
        Dict with all available metadata or None if field doesn't exist

    Example:
        >>> meta = get_field_metadata("status")
        >>> print(meta)
        >>> # {
        >>> #   "type": "option",
        >>> #   "description": "Enable/disable this feature",
        >>> #   "default": "enable",
        >>> #   "options": ["enable", "disable"]
        >>> # }
    """
    if field_name not in FIELD_TYPES:
        return None

    metadata = {
        "name": field_name,
        "type": FIELD_TYPES[field_name],
    }

    # Add description if available
    if field_name in FIELD_DESCRIPTIONS:
        metadata["description"] = FIELD_DESCRIPTIONS[field_name]

    # Add constraints if available
    if field_name in FIELD_CONSTRAINTS:
        metadata["constraints"] = FIELD_CONSTRAINTS[field_name]

    # Add default if available
    if field_name in FIELDS_WITH_DEFAULTS:
        metadata["default"] = FIELDS_WITH_DEFAULTS[field_name]

    # Add required flag
    metadata["required"] = field_name in REQUIRED_FIELDS

    # Add options if available
    options = get_field_options(field_name)
    if options:
        metadata["options"] = options

    # Add nested schema if available
    nested = get_nested_schema(field_name)
    if nested:
        metadata["nested_schema"] = nested

    return metadata


def validate_field_value(field_name: str, value: Any) -> tuple[bool, str | None]:
    """
    Validate a single field value against its constraints.

    Args:
        field_name: Name of the field
        value: Value to validate

    Returns:
        Tuple of (is_valid, error_message)

    Example:
        >>> is_valid, error = validate_field_value("status", "enable")
        >>> if not is_valid:
        ...     print(error)
    """
    # Get field metadata
    field_type = get_field_type(field_name)
    if field_type is None:
        return (False, f"Unknown field: '{field_name}' (not defined in schema)")

    # Get field description for better error context
    description = get_field_description(field_name)

    # Validate enum values
    options = get_field_options(field_name)
    if options and value not in options:
        error_msg = f"Invalid value for '{field_name}': {repr(value)}"
        if description:
            error_msg += f"\n  → Description: {description}"
        error_msg += f"\n  → Valid options: {', '.join(repr(v) for v in options)}"
        if options:
            error_msg += f"\n  → Example: {field_name}={repr(options[0])}"
        return (False, error_msg)

    # Validate constraints
    constraints = get_field_constraints(field_name)
    if constraints:
        constraint_type = constraints.get("type")

        if constraint_type == "integer":
            if not isinstance(value, int):
                error_msg = f"Field '{field_name}' must be an integer"
                if description:
                    error_msg += f"\n  → Description: {description}"
                error_msg += f"\n  → You provided: {type(value).__name__} = {repr(value)}"
                return (False, error_msg)

            min_val = constraints.get("min")
            max_val = constraints.get("max")

            if min_val is not None and value < min_val:
                error_msg = f"Field '{field_name}' value {value} is below minimum {min_val}"
                if description:
                    error_msg += f"\n  → Description: {description}"
                if max_val is not None:
                    error_msg += f"\n  → Valid range: {min_val} to {max_val}"
                return (False, error_msg)

            if max_val is not None and value > max_val:
                error_msg = f"Field '{field_name}' value {value} exceeds maximum {max_val}"
                if description:
                    error_msg += f"\n  → Description: {description}"
                if min_val is not None:
                    error_msg += f"\n  → Valid range: {min_val} to {max_val}"
                return (False, error_msg)

        elif constraint_type == "string":
            if not isinstance(value, str):
                error_msg = f"Field '{field_name}' must be a string"
                if description:
                    error_msg += f"\n  → Description: {description}"
                error_msg += f"\n  → You provided: {type(value).__name__} = {repr(value)}"
                return (False, error_msg)

            max_length = constraints.get("max_length")
            if max_length and len(value) > max_length:
                error_msg = f"Field '{field_name}' length {len(value)} exceeds maximum {max_length}"
                if description:
                    error_msg += f"\n  → Description: {description}"
                error_msg += f"\n  → Your value: {repr(value[:50])}{'...' if len(value) > 50 else ''}"
                return (False, error_msg)

    return (True, None)


# ============================================================================
# Schema Information
# Metadata about this endpoint schema
# ============================================================================

SCHEMA_INFO = {
    "endpoint": "user/radius",
    "category": "cmdb",
    "api_path": "user/radius",
    "mkey": "name",
    "mkey_type": "string",
    "help": "Configure RADIUS server entries.",
    "total_fields": 62,
    "required_fields_count": 5,
    "fields_with_defaults_count": 56,
}


def get_schema_info() -> dict[str, Any]:
    """
    Get information about this endpoint schema.

    Returns:
        Dict with schema metadata

    Example:
        >>> info = get_schema_info()
        >>> print(f"Endpoint: {info['endpoint']}")
        >>> print(f"Total fields: {info['total_fields']}")
    """
    return SCHEMA_INFO.copy()



"""FortiOS CMDB - Language category"""

from .import_setting import ImportSetting

__all__ = [
    "ImportSetting",
    "Language",
]


class Language:
    """Language endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Language endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.import_setting = ImportSetting(client)



"""FortiOS CMDB - Ca category"""

from .import_setting import ImportSetting

__all__ = [
    "Ca",
    "ImportSetting",
]


class Ca:
    """Ca endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Ca endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.import_setting = ImportSetting(client)



from typing import TypedDict, Literal, NotRequired, Any, Coroutine, Union

# Payload TypedDict for IDE autocomplete
class GlobalSettingPayload(TypedDict, total=False):
    """
    Type hints for web_proxy/global_setting payload fields.
    
    Use this for IDE autocomplete when building payload dicts:
        payload: GlobalSettingPayload = {
            "field": "value",  # <- autocomplete shows all fields
        }
    """
    ssl_cert: NotRequired[str]  # SSL certificate for SSL interception.
    ssl_ca_cert: NotRequired[str]  # SSL CA certificate for SSL interception.
    fast_policy_match: NotRequired[Literal["enable", "disable"]]  # Enable/disable fast matching algorithm for explicit and tran
    ldap_user_cache: NotRequired[Literal["enable", "disable"]]  # Enable/disable LDAP user cache for explicit and transparent 
    proxy_fqdn: str  # Fully Qualified Domain Name of the explicit web proxy (defau
    max_request_length: NotRequired[int]  # Maximum length of HTTP request line (2 - 64 Kbytes, default 
    max_message_length: NotRequired[int]  # Maximum length of HTTP message, not including body (16 - 256
    http2_client_window_size: NotRequired[int]  # HTTP/2 client initial window size in bytes (65535 - 21474836
    http2_server_window_size: NotRequired[int]  # HTTP/2 server initial window size in bytes (65535 - 21474836
    strict_web_check: NotRequired[Literal["enable", "disable"]]  # Enable/disable strict web checking to block web sites that s
    forward_proxy_auth: NotRequired[Literal["enable", "disable"]]  # Enable/disable forwarding proxy authentication headers.
    forward_server_affinity_timeout: NotRequired[int]  # Period of time before the source IP's traffic is no longer a
    max_waf_body_cache_length: NotRequired[int]  # Maximum length of HTTP messages processed by Web Application
    webproxy_profile: NotRequired[str]  # Name of the web proxy profile to apply when explicit proxy t
    learn_client_ip: NotRequired[Literal["enable", "disable"]]  # Enable/disable learning the client's IP address from headers
    always_learn_client_ip: NotRequired[Literal["enable", "disable"]]  # Enable/disable learning the client's IP address from headers
    learn_client_ip_from_header: NotRequired[Literal["true-client-ip", "x-real-ip", "x-forwarded-for"]]  # Learn client IP address from the specified headers.
    learn_client_ip_srcaddr: NotRequired[list[dict[str, Any]]]  # Source address name (srcaddr or srcaddr6 must be set).
    learn_client_ip_srcaddr6: NotRequired[list[dict[str, Any]]]  # IPv6 Source address name (srcaddr or srcaddr6 must be set).
    src_affinity_exempt_addr: NotRequired[str]  # IPv4 source addresses to exempt proxy affinity.
    src_affinity_exempt_addr6: NotRequired[str]  # IPv6 source addresses to exempt proxy affinity.
    log_policy_pending: NotRequired[Literal["enable", "disable"]]  # Enable/disable logging sessions that are pending on policy m
    log_forward_server: NotRequired[Literal["enable", "disable"]]  # Enable/disable forward server name logging in forward traffi
    log_app_id: NotRequired[Literal["enable", "disable"]]  # Enable/disable always log application type in traffic log.
    proxy_transparent_cert_inspection: NotRequired[Literal["enable", "disable"]]  # Enable/disable transparent proxy certificate inspection.
    request_obs_fold: NotRequired[Literal["replace-with-sp", "block", "keep"]]  # Action when HTTP/1.x request header contains obs-fold.


class GlobalSetting:
    """
    Configure Web proxy global settings.
    
    Path: web_proxy/global_setting
    Category: cmdb
    """
    
    def get(
        self,
        name: str | None = ...,
        filter: str | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def post(
        self,
        payload_dict: GlobalSettingPayload | None = ...,
        ssl_cert: str | None = ...,
        ssl_ca_cert: str | None = ...,
        fast_policy_match: Literal["enable", "disable"] | None = ...,
        ldap_user_cache: Literal["enable", "disable"] | None = ...,
        proxy_fqdn: str | None = ...,
        max_request_length: int | None = ...,
        max_message_length: int | None = ...,
        http2_client_window_size: int | None = ...,
        http2_server_window_size: int | None = ...,
        strict_web_check: Literal["enable", "disable"] | None = ...,
        forward_proxy_auth: Literal["enable", "disable"] | None = ...,
        forward_server_affinity_timeout: int | None = ...,
        max_waf_body_cache_length: int | None = ...,
        webproxy_profile: str | None = ...,
        learn_client_ip: Literal["enable", "disable"] | None = ...,
        always_learn_client_ip: Literal["enable", "disable"] | None = ...,
        learn_client_ip_from_header: Literal["true-client-ip", "x-real-ip", "x-forwarded-for"] | None = ...,
        learn_client_ip_srcaddr: list[dict[str, Any]] | None = ...,
        learn_client_ip_srcaddr6: list[dict[str, Any]] | None = ...,
        src_affinity_exempt_addr: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def put(
        self,
        payload_dict: GlobalSettingPayload | None = ...,
        ssl_cert: str | None = ...,
        ssl_ca_cert: str | None = ...,
        fast_policy_match: Literal["enable", "disable"] | None = ...,
        ldap_user_cache: Literal["enable", "disable"] | None = ...,
        proxy_fqdn: str | None = ...,
        max_request_length: int | None = ...,
        max_message_length: int | None = ...,
        http2_client_window_size: int | None = ...,
        http2_server_window_size: int | None = ...,
        strict_web_check: Literal["enable", "disable"] | None = ...,
        forward_proxy_auth: Literal["enable", "disable"] | None = ...,
        forward_server_affinity_timeout: int | None = ...,
        max_waf_body_cache_length: int | None = ...,
        webproxy_profile: str | None = ...,
        learn_client_ip: Literal["enable", "disable"] | None = ...,
        always_learn_client_ip: Literal["enable", "disable"] | None = ...,
        learn_client_ip_from_header: Literal["true-client-ip", "x-real-ip", "x-forwarded-for"] | None = ...,
        learn_client_ip_srcaddr: list[dict[str, Any]] | None = ...,
        learn_client_ip_srcaddr6: list[dict[str, Any]] | None = ...,
        src_affinity_exempt_addr: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> Union[bool, Coroutine[Any, Any, bool]]: ...
    
    def set(
        self,
        payload_dict: GlobalSettingPayload | None = ...,
        vdom: str | bool | None = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> Union[list[str], list[dict[str, Any]]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> dict[str, Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> dict[str, Any]: ...
    
    @staticmethod
    def schema() -> dict[str, Any]: ...


__all__ = [
    "GlobalSetting",
    "GlobalSettingPayload",
]


from typing import TypedDict, Literal, NotRequired, Any, Coroutine, Union

# Payload TypedDict for IDE autocomplete
class BgpPayload(TypedDict, total=False):
    """
    Type hints for router/bgp payload fields.
    
    Use this for IDE autocomplete when building payload dicts:
        payload: BgpPayload = {
            "field": "value",  # <- autocomplete shows all fields
        }
    """
    as: str  # Router AS number, asplain/asdot/asdot+ format, 0 to disable 
    router_id: NotRequired[str]  # Router ID.
    keepalive_timer: NotRequired[int]  # Frequency to send keep alive requests.
    holdtime_timer: NotRequired[int]  # Number of seconds to mark peer as dead.
    always_compare_med: NotRequired[Literal["enable", "disable"]]  # Enable/disable always compare MED.
    bestpath_as_path_ignore: NotRequired[Literal["enable", "disable"]]  # Enable/disable ignore AS path.
    bestpath_cmp_confed_aspath: NotRequired[Literal["enable", "disable"]]  # Enable/disable compare federation AS path length.
    bestpath_cmp_routerid: NotRequired[Literal["enable", "disable"]]  # Enable/disable compare router ID for identical EBGP paths.
    bestpath_med_confed: NotRequired[Literal["enable", "disable"]]  # Enable/disable compare MED among confederation paths.
    bestpath_med_missing_as_worst: NotRequired[Literal["enable", "disable"]]  # Enable/disable treat missing MED as least preferred.
    client_to_client_reflection: NotRequired[Literal["enable", "disable"]]  # Enable/disable client-to-client route reflection.
    dampening: NotRequired[Literal["enable", "disable"]]  # Enable/disable route-flap dampening.
    deterministic_med: NotRequired[Literal["enable", "disable"]]  # Enable/disable enforce deterministic comparison of MED.
    ebgp_multipath: NotRequired[Literal["enable", "disable"]]  # Enable/disable EBGP multi-path.
    ibgp_multipath: NotRequired[Literal["enable", "disable"]]  # Enable/disable IBGP multi-path.
    enforce_first_as: NotRequired[Literal["enable", "disable"]]  # Enable/disable enforce first AS for EBGP routes.
    fast_external_failover: NotRequired[Literal["enable", "disable"]]  # Enable/disable reset peer BGP session if link goes down.
    log_neighbour_changes: NotRequired[Literal["enable", "disable"]]  # Log BGP neighbor changes.
    network_import_check: NotRequired[Literal["enable", "disable"]]  # Enable/disable ensure BGP network route exists in IGP.
    ignore_optional_capability: NotRequired[Literal["enable", "disable"]]  # Do not send unknown optional capability notification message
    additional_path: NotRequired[Literal["enable", "disable"]]  # Enable/disable selection of BGP IPv4 additional paths.
    additional_path6: NotRequired[Literal["enable", "disable"]]  # Enable/disable selection of BGP IPv6 additional paths.
    additional_path_vpnv4: NotRequired[Literal["enable", "disable"]]  # Enable/disable selection of BGP VPNv4 additional paths.
    additional_path_vpnv6: NotRequired[Literal["enable", "disable"]]  # Enable/disable selection of BGP VPNv6 additional paths.
    multipath_recursive_distance: NotRequired[Literal["enable", "disable"]]  # Enable/disable use of recursive distance to select multipath
    recursive_next_hop: NotRequired[Literal["enable", "disable"]]  # Enable/disable recursive resolution of next-hop using BGP ro
    recursive_inherit_priority: NotRequired[Literal["enable", "disable"]]  # Enable/disable priority inheritance for recursive resolution
    tag_resolve_mode: NotRequired[Literal["disable", "preferred", "merge", "merge-all"]]  # Configure tag-match mode. Resolves BGP routes with other rou
    cluster_id: NotRequired[str]  # Route reflector cluster ID.
    confederation_identifier: NotRequired[int]  # Confederation identifier.
    confederation_peers: NotRequired[list[dict[str, Any]]]  # Confederation peers.
    dampening_route_map: NotRequired[str]  # Criteria for dampening.
    dampening_reachability_half_life: NotRequired[int]  # Reachability half-life time for penalty (min).
    dampening_reuse: NotRequired[int]  # Threshold to reuse routes.
    dampening_suppress: NotRequired[int]  # Threshold to suppress routes.
    dampening_max_suppress_time: NotRequired[int]  # Maximum minutes a route can be suppressed.
    dampening_unreachability_half_life: NotRequired[int]  # Unreachability half-life time for penalty (min).
    default_local_preference: NotRequired[int]  # Default local preference.
    scan_time: NotRequired[int]  # Background scanner interval (sec), 0 to disable it.
    distance_external: NotRequired[int]  # Distance for routes external to the AS.
    distance_internal: NotRequired[int]  # Distance for routes internal to the AS.
    distance_local: NotRequired[int]  # Distance for routes local to the AS.
    synchronization: NotRequired[Literal["enable", "disable"]]  # Enable/disable only advertise routes from iBGP if routes pre
    graceful_restart: NotRequired[Literal["enable", "disable"]]  # Enable/disable BGP graceful restart capabilities.
    graceful_restart_time: NotRequired[int]  # Time needed for neighbors to restart (sec).
    graceful_stalepath_time: NotRequired[int]  # Time to hold stale paths of restarting neighbor (sec).
    graceful_update_delay: NotRequired[int]  # Route advertisement/selection delay after restart (sec).
    graceful_end_on_timer: NotRequired[Literal["enable", "disable"]]  # Enable/disable to exit graceful restart on timer only.
    additional_path_select: NotRequired[int]  # Number of additional paths to be selected for each IPv4 NLRI
    additional_path_select6: NotRequired[int]  # Number of additional paths to be selected for each IPv6 NLRI
    additional_path_select_vpnv4: NotRequired[int]  # Number of additional paths to be selected for each VPNv4 NLR
    additional_path_select_vpnv6: NotRequired[int]  # Number of additional paths to be selected for each VPNv6 NLR
    cross_family_conditional_adv: NotRequired[Literal["enable", "disable"]]  # Enable/disable cross address family conditional advertisemen
    aggregate_address: NotRequired[list[dict[str, Any]]]  # BGP aggregate address table.
    aggregate_address6: NotRequired[list[dict[str, Any]]]  # BGP IPv6 aggregate address table.
    neighbor: NotRequired[list[dict[str, Any]]]  # BGP neighbor table.
    neighbor_group: NotRequired[list[dict[str, Any]]]  # BGP neighbor group table.
    neighbor_range: NotRequired[list[dict[str, Any]]]  # BGP neighbor range table.
    neighbor_range6: NotRequired[list[dict[str, Any]]]  # BGP IPv6 neighbor range table.
    network: NotRequired[list[dict[str, Any]]]  # BGP network table.
    network6: NotRequired[list[dict[str, Any]]]  # BGP IPv6 network table.
    redistribute: NotRequired[list[dict[str, Any]]]  # BGP IPv4 redistribute table.
    redistribute6: NotRequired[list[dict[str, Any]]]  # BGP IPv6 redistribute table.
    admin_distance: NotRequired[list[dict[str, Any]]]  # Administrative distance modifications.
    vrf: NotRequired[list[dict[str, Any]]]  # BGP VRF leaking table.
    vrf6: NotRequired[list[dict[str, Any]]]  # BGP IPv6 VRF leaking table.


class Bgp:
    """
    Configure BGP.
    
    Path: router/bgp
    Category: cmdb
    """
    
    def get(
        self,
        name: str | None = ...,
        filter: str | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def post(
        self,
        payload_dict: BgpPayload | None = ...,
        as: str | None = ...,
        router_id: str | None = ...,
        keepalive_timer: int | None = ...,
        holdtime_timer: int | None = ...,
        always_compare_med: Literal["enable", "disable"] | None = ...,
        bestpath_as_path_ignore: Literal["enable", "disable"] | None = ...,
        bestpath_cmp_confed_aspath: Literal["enable", "disable"] | None = ...,
        bestpath_cmp_routerid: Literal["enable", "disable"] | None = ...,
        bestpath_med_confed: Literal["enable", "disable"] | None = ...,
        bestpath_med_missing_as_worst: Literal["enable", "disable"] | None = ...,
        client_to_client_reflection: Literal["enable", "disable"] | None = ...,
        dampening: Literal["enable", "disable"] | None = ...,
        deterministic_med: Literal["enable", "disable"] | None = ...,
        ebgp_multipath: Literal["enable", "disable"] | None = ...,
        ibgp_multipath: Literal["enable", "disable"] | None = ...,
        enforce_first_as: Literal["enable", "disable"] | None = ...,
        fast_external_failover: Literal["enable", "disable"] | None = ...,
        log_neighbour_changes: Literal["enable", "disable"] | None = ...,
        network_import_check: Literal["enable", "disable"] | None = ...,
        ignore_optional_capability: Literal["enable", "disable"] | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def put(
        self,
        payload_dict: BgpPayload | None = ...,
        as: str | None = ...,
        router_id: str | None = ...,
        keepalive_timer: int | None = ...,
        holdtime_timer: int | None = ...,
        always_compare_med: Literal["enable", "disable"] | None = ...,
        bestpath_as_path_ignore: Literal["enable", "disable"] | None = ...,
        bestpath_cmp_confed_aspath: Literal["enable", "disable"] | None = ...,
        bestpath_cmp_routerid: Literal["enable", "disable"] | None = ...,
        bestpath_med_confed: Literal["enable", "disable"] | None = ...,
        bestpath_med_missing_as_worst: Literal["enable", "disable"] | None = ...,
        client_to_client_reflection: Literal["enable", "disable"] | None = ...,
        dampening: Literal["enable", "disable"] | None = ...,
        deterministic_med: Literal["enable", "disable"] | None = ...,
        ebgp_multipath: Literal["enable", "disable"] | None = ...,
        ibgp_multipath: Literal["enable", "disable"] | None = ...,
        enforce_first_as: Literal["enable", "disable"] | None = ...,
        fast_external_failover: Literal["enable", "disable"] | None = ...,
        log_neighbour_changes: Literal["enable", "disable"] | None = ...,
        network_import_check: Literal["enable", "disable"] | None = ...,
        ignore_optional_capability: Literal["enable", "disable"] | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> Union[bool, Coroutine[Any, Any, bool]]: ...
    
    def set(
        self,
        payload_dict: BgpPayload | None = ...,
        vdom: str | bool | None = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> Union[list[str], list[dict[str, Any]]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> dict[str, Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> dict[str, Any]: ...
    
    @staticmethod
    def schema() -> dict[str, Any]: ...


__all__ = [
    "Bgp",
    "BgpPayload",
]


from typing import TypedDict, Literal, NotRequired, Any, Coroutine, Union


class FortiguardCategories:
    """
    Monitor endpoint for videofilter/fortiguard-categories
    
    Path: videofilter/fortiguard_categories
    Category: monitor
    """
    
    def get(
        self,
        name: str | None = ...,
        filter: str | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def post(
        self,
        payload_dict: dict[str, Any] | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def put(
        self,
        payload_dict: dict[str, Any] | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> Union[bool, Coroutine[Any, Any, bool]]: ...
    
    def set(
        self,
        payload_dict: dict[str, Any] | None = ...,
        vdom: str | bool | None = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> Union[list[str], list[dict[str, Any]]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> dict[str, Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> dict[str, Any]: ...
    
    @staticmethod
    def schema() -> dict[str, Any]: ...


__all__ = [
    "FortiguardCategories",
]


from typing import TypedDict, Literal, NotRequired, Any, Coroutine, Union

# Payload TypedDict for IDE autocomplete
class SettingPayload(TypedDict, total=False):
    """
    Type hints for log/tacacs_plus_accounting3/setting payload fields.
    
    Use this for IDE autocomplete when building payload dicts:
        payload: SettingPayload = {
            "field": "value",  # <- autocomplete shows all fields
        }
    """
    status: NotRequired[Literal["enable", "disable"]]  # Enable/disable TACACS+ accounting.
    server: str  # Address of TACACS+ server.
    server_key: NotRequired[str]  # Key to access the TACACS+ server.
    source_ip: NotRequired[str]  # Source IP address for communication to TACACS+ server.
    interface_select_method: NotRequired[Literal["auto", "sdwan", "specify"]]  # Specify how to select outgoing interface to reach server.
    interface: str  # Specify outgoing interface to reach server.
    vrf_select: NotRequired[int]  # VRF ID used for connection to server.


class Setting:
    """
    Settings for TACACS+ accounting.
    
    Path: log/tacacs_plus_accounting3/setting
    Category: cmdb
    """
    
    def get(
        self,
        name: str | None = ...,
        filter: str | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def post(
        self,
        payload_dict: SettingPayload | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        server: str | None = ...,
        server_key: str | None = ...,
        source_ip: str | None = ...,
        interface_select_method: Literal["auto", "sdwan", "specify"] | None = ...,
        interface: str | None = ...,
        vrf_select: int | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def put(
        self,
        payload_dict: SettingPayload | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        server: str | None = ...,
        server_key: str | None = ...,
        source_ip: str | None = ...,
        interface_select_method: Literal["auto", "sdwan", "specify"] | None = ...,
        interface: str | None = ...,
        vrf_select: int | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        raw_json: bool = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> Union[bool, Coroutine[Any, Any, bool]]: ...
    
    def set(
        self,
        payload_dict: SettingPayload | None = ...,
        vdom: str | bool | None = ...,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> Union[list[str], list[dict[str, Any]]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> dict[str, Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> dict[str, Any]: ...
    
    @staticmethod
    def schema() -> dict[str, Any]: ...


__all__ = [
    "Setting",
    "SettingPayload",
]